@@ -762,7 +762,7 @@ static void* System_launchParallelTasksThread(void *in)
     pthread_mutex_lock(&data->mutex);
     n = data->current++;
     pthread_mutex_unlock(&data->mutex);
-    if (data->fail || data->current > data->len) break;
+    if (data->fail || n >= data->len) break;
     MMC_TRY_TOP()
     threadData->parent = data->parent;
     threadData->mmc_thread_work_exit = threadData->mmc_jumper;
@@ -790,24 +790,23 @@ extern void* System_launchParallelTasks(threadData_t *threadData, int numThreads
 {
   int len = listLength(dataLst), i;
   void *result = mmc_mk_nil();
-  thread_data data;
+  thread_data data = {0};
 #if !defined(_MSC_VER)
   void *commands[len];
   void *status[len];
-  int ids[len];
   pthread_t th[numThreads];
+  int isInteger = 0;
 
 #if defined(__MINGW32__)
-  /* adrpo: set thread stack size on Windows to 2MB */
+  /* adrpo: set thread stack size on Windows to 4MB */
   pthread_attr_t attr;
   pthread_attr_init(&attr);
-  pthread_attr_setstacksize(&attr, 2097152);
+  pthread_attr_setstacksize(&attr, 4194304);
 #endif
 
-#else
+#else /* MSVC */
   void **commands = (void**) omc_alloc_interface.malloc(sizeof(void*)*len);
   void **status = (void**) omc_alloc_interface.malloc(sizeof(void*)*len);
-  int *ids = (int*) omc_alloc_interface.malloc_atomic(sizeof(int)*len);
   pthread_t *th = (pthread_t*) omc_alloc_interface.malloc(sizeof(pthread_t)*numThreads);
 #endif
   if (len == 0) {
@@ -815,6 +814,12 @@ extern void* System_launchParallelTasks(threadData_t *threadData, int numThreads
   } else if (numThreads == 1 || len == 1) {
     return System_launchParallelTasksSerial(threadData,dataLst,fn);
   }
+
+  /* Make sure we get nothing unexpected here */
+  memset(commands, 0, len*sizeof(void*));
+  memset(status, 0, len*sizeof(void*));
+  memset(th, 0, numThreads*sizeof(pthread_t));
+
   pthread_mutex_init(&data.mutex,NULL);
   data.fn = fn;
   data.current = 0;
@@ -829,19 +834,60 @@ extern void* System_launchParallelTasks(threadData_t *threadData, int numThreads
   }
   numThreads = numThreads > len ? len : numThreads;
   for (i=0; i<numThreads; i++) {
+    if (GC_pthread_create(&th[i],
 #if defined(__MINGW32__)
-    GC_pthread_create(&th[i],&attr,System_launchParallelTasksThread,&data);
+    &attr,
 #else
-    GC_pthread_create(&th[i],NULL,System_launchParallelTasksThread,&data);
+    NULL,
 #endif
+    System_launchParallelTasksThread,&data)) {
+      /* GC_pthread_create failed. We need to join already created threads though... */
+      const char *tok[1] = {strerror(errno)};
+      data.fail = 1;
+      c_add_message(NULL,5999,
+        ErrorType_scripting,
+        ErrorLevel_internal,
+        gettext("System.launchParallelTasks: Failed to create thread: %s"),
+        NULL,
+        0);
+      break;
+    }
   }
   for (i=0; i<numThreads; i++) {
-    GC_pthread_join(th[i], NULL);
+    if (th[i] && GC_pthread_join(th[i], NULL)) {
+      const char *tok[1] = {strerror(errno)};
+      data.fail = 1;
+      c_add_message(NULL,5999,
+        ErrorType_scripting,
+        ErrorLevel_internal,
+        gettext("System.launchParallelTasks: Failed to join thread: %s"),
+        NULL,
+        0);
+    }
   }
   if (data.fail) {
     MMC_THROW_INTERNAL();
   }
+  if (data.current < len) {
+    c_add_message(NULL,5999,
+      ErrorType_scripting,
+      ErrorLevel_internal,
+      gettext("System.launchParallelTasks: We seem to have executed fewer tasks than expected."),
+      NULL,
+      0);
+    MMC_THROW_INTERNAL();
+  }
+  isInteger = MMC_IS_INTEGER(status[0]);
   for (i=len-1; i>=0; i--) {
+    if (isInteger != MMC_IS_INTEGER(status[i])) {
+      c_add_message(NULL,5999,
+        ErrorType_scripting,
+        ErrorLevel_internal,
+        gettext("System.launchParallelTasks: Got mismatched results types. Was there a thread synchronization error?"),
+        NULL,
+        0);
+      MMC_THROW_INTERNAL();
+    }
     result = mmc_mk_cons(status[i], result);
   }
   return result;
