@@ -134,17 +134,17 @@ int pickUpModelData(DATA* data, threadData_t *threadData, SOLVER_INFO* solverInf
  * author: Vitalij Ruge
  */
 static inline void pickUpDim(OptDataDim * dim, DATA* data, OptDataTime * time){
-
   char * cflags = NULL;
   cflags = (char*)omc_flagValue[FLAG_OPTIMIZER_NP];
-  if(cflags){
+  if (cflags) {
     dim->np = atoi(cflags);
-    if(dim->np != 1 && dim->np!=3){
+    if (dim->np != 1 && dim->np!=3) {
       warningStreamPrint(LOG_STDOUT, 0, "FLAG_OPTIZER_NP is %i. Currently optimizer support only 1 and 3.\nFLAG_OPTIZER_NP set of 3", dim->np);
       dim->np = 3;
     }
-  }else
+  } else {
     dim->np = 3; /*ToDo*/
+  }
   dim->nx = data->modelData->nStates;
   dim->nu = data->modelData->nInputVars;
   dim->nv = dim->nx + dim->nu;
@@ -155,14 +155,17 @@ static inline void pickUpDim(OptDataDim * dim, DATA* data, OptDataTime * time){
   dim->nReal = data->modelData->nVariablesReal;
 
   cflags = (char*)omc_flagValue[FLAG_OPTIMIZER_TGRID];
+  dim->nsi = -1; /* Initialize the data just in case */
   data->callback->getTimeGrid(data, &dim->nsi, &time->tt); /* TODO: dim->nsi is long*, expected is int* */
   time->model_grid = (modelica_boolean)(dim->nsi > 0);
 
-  if(!time->model_grid)
+  if (!time->model_grid) {
     dim->nsi = data->simulationInfo->numSteps;
+  }
 
-  if(cflags)
+  if (cflags) {
     dim->nsi = getNsi(cflags, dim->nsi, &dim->exTimeGrid);
+  }
 
   dim->nt = dim->nsi*dim->np;
   dim->NV = dim->nt*dim->nv;
@@ -320,7 +323,7 @@ int cmp_modelica_real(const void *v1, const void *v2) {
 
 static inline void overwriteTimeGridModel(OptDataTime * time, long double c[], const int np, const int nsi){
   int i,k;
-  const int np1 = np - 1;
+
   time->t0 = time->tt[0];
   time->tf = time->tt[nsi];
 
@@ -691,7 +694,7 @@ void optData2ModelData(OptData *optData, double *vopt, const int index){
 
 
   modelica_real * realVars[3];
-  modelica_real * tmpVars[2];
+  modelica_real * tmpVars[2] = {NULL, NULL};
 
   int i, j, k, shift, l;
   DATA * data = optData->data;
@@ -816,7 +819,7 @@ void diffSynColoredOptimizerSystem(OptData *optData, modelica_real **J, const in
   const int h_index = optData->s.indexABCD[index];
   const long double * scaldt = optData->bounds.scaldt[m];
   const unsigned int * const cC = data->simulationInfo->analyticJacobians[h_index].sparsePattern.colorCols;
-  const unsigned int * const lindex  = optData->s.lindex[index];
+  const unsigned int * const lindex  = data->simulationInfo->analyticJacobians[h_index].sparsePattern.leadindex;
   const int nx = data->simulationInfo->analyticJacobians[h_index].sizeCols;
   const int Cmax = data->simulationInfo->analyticJacobians[h_index].sparsePattern.maxColors + 1;
   const int dnx = optData->dim.nx;
@@ -869,7 +872,7 @@ void diffSynColoredOptimizerSystemF(OptData *optData, modelica_real **J){
     const int index = 4;
     const int h_index = optData->s.indexABCD[index];
     const unsigned int * const cC = data->simulationInfo->analyticJacobians[h_index].sparsePattern.colorCols;
-    const unsigned int * const lindex  = optData->s.lindex[index];
+    const unsigned int * const lindex  = data->simulationInfo->analyticJacobians[h_index].sparsePattern.leadindex;
     const int nx = data->simulationInfo->analyticJacobians[h_index].sizeCols;
     const int Cmax = data->simulationInfo->analyticJacobians[h_index].sparsePattern.maxColors + 1;
     const modelica_real * const resultVars = data->simulationInfo->analyticJacobians[h_index].resultVars;
@@ -917,7 +920,7 @@ static inline void pickUpStates(OptData* optData){
       }
       // check if csv file is empty!
       if(n == 0){
-        fprintf(stderr, "External input file: externalInput.csv is empty!\n"); fflush(NULL);
+        fprintf(stderr, "External input file: %s is empty!\n",cflags); fflush(NULL);
         EXIT(1);
       }else{
         int i, j;
