@@ -45,9 +45,9 @@
 #include "util/list.h"
 
 #define omc_dummyVarInfo {-1,-1,"","",omc_dummyFileInfo}
-#define omc_dummyEquationInfo {-1,0,"",-1,NULL}
+#define omc_dummyEquationInfo {-1,0,0,-1,NULL}
 #define omc_dummyFunctionInfo {-1,"",omc_dummyFileInfo}
-#define omc_dummyRealAttribute {NULL,NULL,DBL_MAX,-DBL_MAX,0,0,1.0,0,0.0}
+#define omc_dummyRealAttribute {NULL,NULL,-DBL_MAX,DBL_MAX,0,0,1.0,0.0}
 
 #if defined(_MSC_VER)
 #define set_struct(TYPE, x, info) { const TYPE tmp = info; x = tmp; }
@@ -269,10 +269,15 @@ typedef struct NONLINEAR_SYSTEM_DATA
   int (*initialAnalyticalJacobian)(void*, threadData_t*);
   modelica_integer jacobianIndex;
 
+  SPARSE_PATTERN sparsePattern;        /* sparse pattern if no jacobian is available */
+  modelica_boolean isPatternAvailable;
+
   void (*residualFunc)(void**, const double*, double*, const int*);
+  int (*residualFuncConstraints)(void**, const double*, double*, const int*);
   void (*initializeStaticNLSData)(void*, threadData_t *threadData, void*);
   int (*strictTearingFunctionCall)(struct DATA*, threadData_t *threadData);
   void (*getIterationVars)(struct DATA*, double*);
+  int (*checkConstraints)(struct DATA*, threadData_t *threadData);
 
   void *solverData;
   modelica_real *nlsx;                 /* x */
@@ -284,15 +289,18 @@ typedef struct NONLINEAR_SYSTEM_DATA
 
   modelica_real residualError;         /* not used */
   modelica_boolean solved;             /* 1: solved in current step - else not */
-  modelica_real lastTimeSolved;         /* save last successful solved point in time */
+  modelica_real lastTimeSolved;        /* save last successful solved point in time */
 
   /* statistics */
-  unsigned long numberOfCall;           /* number of solving calls of this system */
-  unsigned long numberOfFEval;          /* number of function evaluations of this system */
-  unsigned long numberOfIterations;     /* number of iteration of non-linear solvers of this system */
-  double totalTime;                     /* save the totalTime */
-  rtclock_t totalTimeClock;             /* time clock for the totalTime  */
-  void* csvData;                        /* information to save csv data */
+  unsigned long numberOfCall;          /* number of solving calls of this system */
+  unsigned long numberOfFEval;         /* number of function evaluations of this system */
+  unsigned long numberOfJEval;         /* number of jacobian evaluations of this system */
+  unsigned long numberOfIterations;    /* number of iteration of non-linear solvers of this system */
+  double totalTime;                    /* save the totalTime */
+  rtclock_t totalTimeClock;            /* time clock for the totalTime  */
+  double jacobianTime;                 /* save the time to calculate jacobians */
+  rtclock_t jacobianTimeClock;         /* time clock for the jacobianTime  */
+  void* csvData;                       /* information to save csv data */
 } NONLINEAR_SYSTEM_DATA;
 #else
 typedef void* NONLINEAR_SYSTEM_DATA;
@@ -316,6 +324,7 @@ typedef struct LINEAR_SYSTEM_DATA
   void (*residualFunc)(void**, const double*, double*, const int*);
   void (*initializeStaticLSData)(void*, threadData_t *threadData, void*);
   int (*strictTearingFunctionCall)(struct DATA*, threadData_t *threadData);
+  int (*checkConstraints)(struct DATA*, threadData_t *threadData);
 
   /* attributes of iteration variables */
   modelica_real *min;
@@ -339,8 +348,10 @@ typedef struct LINEAR_SYSTEM_DATA
 
   /* statistics */
   unsigned long numberOfCall;           /* number of solving calls of this system */
+  unsigned long numberOfJEval;          /* number of jacobian evaluations of this system */
   double totalTime;                     /* save the totalTime */
   rtclock_t totalTimeClock;             /* time clock for the totalTime  */
+  double jacobianTime;                  /* save the time to calculate jacobians */
 }LINEAR_SYSTEM_DATA;
 #else
 typedef void* LINEAR_SYSTEM_DATA;
@@ -426,6 +437,15 @@ typedef struct DAEMODE_DATA
 
 } DAEMODE_DATA;
 
+typedef struct INLINE_DATA
+{
+  /* step size for inline step */
+  modelica_real dt;
+  /* alg-state variables */
+  modelica_real* algVars;
+  /* old alg-state variables */
+  modelica_real* algOldVars;
+} INLINE_DATA;
 
 typedef struct MODEL_DATA_XML
 {
@@ -640,6 +660,8 @@ typedef struct SIMULATION_INFO
 
   DAEMODE_DATA* daeModeData;
 
+  INLINE_DATA* inlineData;
+
   /* delay vars */
   double tStart;
   RINGBUFFER **delayStructure;
