@@ -29,17 +29,19 @@
  *
  */
 /*
- *
  * @author Adeel Asghar <adeel.asghar@liu.se>
- *
- * RCS: $Id$
- *
  */
 
 #include "ShapeAnnotation.h"
-#include "ModelWidgetContainer.h"
+#include "Util/Helper.h"
+#include "MainWindow.h"
+#include "Options/OptionsDialog.h"
+#include "Modeling/ModelWidgetContainer.h"
 #include "ShapePropertiesDialog.h"
-#include "Commands.h"
+#include "Modeling/Commands.h"
+#include "Component/ComponentProperties.h"
+#include "TLM/FetchInterfaceDataDialog.h"
+#include "Plotting/VariablesWidget.h"
 
 /*!
  * \brief GraphicItem::setDefaults
@@ -62,6 +64,7 @@ void GraphicItem::setDefaults(ShapeAnnotation *pShapeAnnotation)
   mVisible = pShapeAnnotation->mVisible;
   mOrigin = pShapeAnnotation->mOrigin;
   mRotation = pShapeAnnotation->mRotation;
+  mDynamicVisible = pShapeAnnotation->mDynamicVisible;
 }
 
 /*!
@@ -75,7 +78,17 @@ void GraphicItem::parseShapeAnnotation(QString annotation)
   if (list.size() < 3)
     return;
   // if first item of list is true then the shape should be visible.
-  mVisible = list.at(0).contains("true");
+  if (list.at(0).startsWith("{")) {
+    // DynamicSelect
+    QStringList args = StringHandler::getStrings(StringHandler::removeFirstLastCurlBrackets(list.at(0)));
+    if (args.count() > 0)
+      mVisible = args.at(0).contains("true");
+    if (args.count() > 1)
+      mDynamicVisible = args.at(1);  // variable name
+  }
+  else {
+    mVisible = list.at(0).contains("true");
+  }
   // 2nd item is the origin
   QStringList originList = StringHandler::getStrings(StringHandler::removeFirstLastCurlBrackets(list.at(1)));
   if (originList.size() >= 2)
@@ -303,6 +316,7 @@ ShapeAnnotation::ShapeAnnotation(bool inheritedShape, GraphicsView *pGraphicsVie
   : QGraphicsItem(pParent)
 {
   mpGraphicsView = pGraphicsView;
+  mpParentComponent = 0;
   mTransformation = Transformation(StringHandler::Diagram);
   mIsCustomShape = true;
   mIsInheritedShape = inheritedShape;
@@ -334,7 +348,7 @@ void ShapeAnnotation::setDefaults()
   mOriginalTextString = "";
   mTextString = "";
   mFontSize = 0;
-  mFontName = "";
+  mFontName = Helper::systemFontInfo.family();
   mHorizontalAlignment = StringHandler::TextAlignmentCenter;
   mOriginalFileName = "";
   mFileName = "";
@@ -374,6 +388,7 @@ void ShapeAnnotation::setDefaults(ShapeAnnotation *pShapeAnnotation)
   mClassFileName = pShapeAnnotation->mClassFileName;
   mImageSource = pShapeAnnotation->mImageSource;
   mImage = pShapeAnnotation->mImage;
+  mDynamicTextString = pShapeAnnotation->mDynamicTextString;
 }
 
 /*!
@@ -382,10 +397,9 @@ void ShapeAnnotation::setDefaults(ShapeAnnotation *pShapeAnnotation)
   */
 void ShapeAnnotation::setUserDefaults()
 {
-  OptionsDialog *pOptionsDialog = mpGraphicsView->getModelWidget()->getModelWidgetContainer()->getMainWindow()->getOptionsDialog();
+  OptionsDialog *pOptionsDialog = OptionsDialog::instance();
   /* Set user Line Style settings */
-  if (pOptionsDialog->getLineStylePage()->getLineColor().isValid())
-  {
+  if (pOptionsDialog->getLineStylePage()->getLineColor().isValid()) {
     mLineColor = pOptionsDialog->getLineStylePage()->getLineColor();
   }
   mLinePattern = StringHandler::getLinePatternType(pOptionsDialog->getLineStylePage()->getLinePattern());
@@ -393,17 +407,13 @@ void ShapeAnnotation::setUserDefaults()
   mArrow.replace(0, StringHandler::getArrowType(pOptionsDialog->getLineStylePage()->getLineStartArrow()));
   mArrow.replace(1, StringHandler::getArrowType(pOptionsDialog->getLineStylePage()->getLineEndArrow()));
   mArrowSize = pOptionsDialog->getLineStylePage()->getLineArrowSize();
-  if (pOptionsDialog->getLineStylePage()->getLineSmooth())
-  {
+  if (pOptionsDialog->getLineStylePage()->getLineSmooth()) {
     mSmooth = StringHandler::SmoothBezier;
-  }
-  else
-  {
+  } else {
     mSmooth = StringHandler::SmoothNone;
   }
   /* Set user Fill Style settings */
-  if (pOptionsDialog->getFillStylePage()->getFillColor().isValid())
-  {
+  if (pOptionsDialog->getFillStylePage()->getFillColor().isValid()) {
     mFillColor = pOptionsDialog->getFillStylePage()->getFillColor();
   }
   mFillPattern = StringHandler::getFillPatternType(pOptionsDialog->getFillStylePage()->getFillPattern());
@@ -415,14 +425,27 @@ bool ShapeAnnotation::isInheritedShape()
 }
 
 /*!
-  Defines the actions used by the shape's context menu.
-  */
+ * \brief ShapeAnnotation::createActions
+ * Defines the actions used by the shape's context menu.
+ */
 void ShapeAnnotation::createActions()
 {
   // shape properties
   mpShapePropertiesAction = new QAction(Helper::properties, mpGraphicsView);
   mpShapePropertiesAction->setStatusTip(tr("Shows the shape properties"));
   connect(mpShapePropertiesAction, SIGNAL(triggered()), SLOT(showShapeProperties()));
+  // shape attributes
+  mpAlignInterfacesAction = new QAction(QIcon(":/Resources/icons/align-interfaces.svg"), Helper::alignInterfaces, mpGraphicsView);
+  mpAlignInterfacesAction->setStatusTip(Helper::alignInterfacesTip);
+  connect(mpAlignInterfacesAction, SIGNAL(triggered()), SLOT(alignInterfaces()));
+  // shape attributes
+  mpShapeAttributesAction = new QAction(Helper::attributes, mpGraphicsView);
+  mpShapeAttributesAction->setStatusTip(tr("Shows the shape attributes"));
+  connect(mpShapeAttributesAction, SIGNAL(triggered()), SLOT(showShapeAttributes()));
+  // edit transition action
+  mpEditTransitionAction = new QAction(Helper::editTransition, mpGraphicsView);
+  mpEditTransitionAction->setStatusTip(tr("Edits the transition"));
+  connect(mpEditTransitionAction, SIGNAL(triggered()), SLOT(editTransition()));
 }
 
 /*!
@@ -454,14 +477,21 @@ QRectF ShapeAnnotation::getBoundingRect() const
 }
 
 /*!
-  Applies the shape line pattern.
-  \param painter - pointer to QPainter
-  */
+ * \brief ShapeAnnotation::applyLinePattern
+ * Applies the shape line pattern.
+ * \param painter - pointer to QPainter
+ */
 void ShapeAnnotation::applyLinePattern(QPainter *painter)
 {
-  qreal thicknessFactor = mLineThickness / 0.5;
-  qreal thickness = thicknessFactor < 1 ? 1.0 : thicknessFactor;
+  qreal thickness = Utilities::convertMMToPixel(mLineThickness);
   QPen pen(mLineColor, thickness, StringHandler::getLinePatternType(mLinePattern), Qt::SquareCap, Qt::MiterJoin);
+  /* The specification doesn't say anything about it.
+   * But just to keep this consist with Dymola we use Qt::BevelJoin for Line shapes.
+   * All other shapes use Qt::MiterJoin
+   */
+  if (dynamic_cast<LineAnnotation*>(this)) {
+    pen.setJoinStyle(Qt::BevelJoin);
+  }
   /* Ticket #3222
    * Make all the shapes use cosmetic pens so that they perserve their pen widht when scaled i.e zoomed in/out.
    * Only shapes with border patterns raised & sunken don't use cosmetic pens. We need better handling of border patterns.
@@ -469,15 +499,15 @@ void ShapeAnnotation::applyLinePattern(QPainter *painter)
   if (mBorderPattern != StringHandler::BorderRaised && mBorderPattern != StringHandler::BorderSunken) {
     pen.setCosmetic(true);
   }
-  if (mpGraphicsView && mpGraphicsView->isRenderingLibraryPixmap()) {
-    /* Ticket #2272, Ticket #2268.
-     * If thickness is greater than 2 then don't make the pen cosmetic since cosmetic pens don't change the width with respect to zoom.
-     */
-    if (thickness <= 2) {
-      pen.setCosmetic(true);
-    } else {
-      pen.setCosmetic(false);
-    }
+  /* Ticket #2272, Ticket #2268.
+   * If thickness is greater than 2 then don't make the pen cosmetic since cosmetic pens don't change the width with respect to zoom.
+   */
+  if (mpGraphicsView && mpGraphicsView->isRenderingLibraryPixmap() && thickness > 2) {
+    pen.setCosmetic(false);
+  }
+  // if thickness is greater than 1 pixel then use antialiasing.
+  if (thickness > 1) {
+    painter->setRenderHint(QPainter::Antialiasing);
   }
   painter->setPen(pen);
 }
@@ -488,38 +518,35 @@ void ShapeAnnotation::applyLinePattern(QPainter *painter)
   */
 void ShapeAnnotation::applyFillPattern(QPainter *painter)
 {
-  switch (mFillPattern)
-  {
+  QRectF boundingRectangle;
+  if (dynamic_cast<PolygonAnnotation*>(this)) {
+    boundingRectangle = boundingRect();
+  } else {
+    boundingRectangle = getBoundingRect();
+  }
+  QLinearGradient linearGradient;
+  QRadialGradient radialGradient;
+  switch (mFillPattern) {
     case StringHandler::FillHorizontalCylinder:
-    {
-      QLinearGradient gradient(getBoundingRect().center().x(), getBoundingRect().center().y(),
-                               getBoundingRect().center().x(), getBoundingRect().y());
-      gradient.setColorAt(0.0, mFillColor);
-      gradient.setColorAt(1.0, mLineColor);
-      gradient.setSpread(QGradient::ReflectSpread);
-      painter->setBrush(gradient);
+      linearGradient = QLinearGradient(boundingRectangle.center().x(), boundingRectangle.top(), boundingRectangle.center().x(), boundingRectangle.bottom());
+      linearGradient.setColorAt(0.0, mLineColor);
+      linearGradient.setColorAt(0.5, mFillColor);
+      linearGradient.setColorAt(1.0, mLineColor);
+      painter->setBrush(linearGradient);
       break;
-    }
     case StringHandler::FillVerticalCylinder:
-    {
-      QLinearGradient gradient(getBoundingRect().center().x(), getBoundingRect().center().y(),
-                               getBoundingRect().x(), getBoundingRect().center().y());
-      gradient.setColorAt(0.0, mFillColor);
-      gradient.setColorAt(1.0, mLineColor);
-      gradient.setSpread(QGradient::ReflectSpread);
-      painter->setBrush(gradient);
+      linearGradient = QLinearGradient(boundingRectangle.left(), boundingRectangle.center().y(), boundingRectangle.right(), boundingRectangle.center().y());
+      linearGradient.setColorAt(0.0, mLineColor);
+      linearGradient.setColorAt(0.5, mFillColor);
+      linearGradient.setColorAt(1.0, mLineColor);
+      painter->setBrush(linearGradient);
       break;
-    }
     case StringHandler::FillSphere:
-    {
-      QRadialGradient gradient(getBoundingRect().center().x(), getBoundingRect().center().y(),
-                               getBoundingRect().width());
-      gradient.setColorAt(0.0, mFillColor);
-      gradient.setColorAt(1.0, mLineColor);
-      //gradient.setSpread(QGradient::ReflectSpread);
-      painter->setBrush(gradient);
+      radialGradient = QRadialGradient(boundingRectangle.center().x(), boundingRectangle.center().y(), boundingRectangle.width());
+      radialGradient.setColorAt(0.0, mFillColor);
+      radialGradient.setColorAt(1.0, mLineColor);
+      painter->setBrush(radialGradient);
       break;
-    }
     case StringHandler::FillSolid:
       painter->setBrush(QBrush(mFillColor, StringHandler::getFillPatternType(mFillPattern)));
       break;
@@ -596,8 +623,11 @@ void ShapeAnnotation::drawCornerItems()
     for (int i = 0 ; i < mPoints.size() ; i++) {
       QPointF point = mPoints.at(i);
       CornerItem *pCornerItem = new CornerItem(point.x(), point.y(), i, this);
-      /* if line is a connection then make the first and last point non moveable. */
-      if ((lineType == LineAnnotation::ConnectionType) && (i == 0 || i == mPoints.size() - 1)) {
+      /* if line is a connection or transition then make the first and last point non movable.
+       * if line is initial state then make the first point non movable.
+       */
+      if ((((lineType == LineAnnotation::ConnectionType || lineType == LineAnnotation::TransitionType) && (i == 0 || i == mPoints.size() - 1))
+           || (lineType == LineAnnotation::InitialStateType && i == 0))) {
         pCornerItem->setFlag(QGraphicsItem::ItemIsMovable, false);
       }
       mCornerItemsList.append(pCornerItem);
@@ -679,9 +709,8 @@ void ShapeAnnotation::setTextString(QString textString)
  * \brief ShapeAnnotation::setFileName
  * Sets the file name.
  * \param fileName
- * \param pComponent
  */
-void ShapeAnnotation::setFileName(QString fileName, Component *pComponent)
+void ShapeAnnotation::setFileName(QString fileName)
 {
   if (fileName.isEmpty()) {
     mOriginalFileName = fileName;
@@ -689,18 +718,11 @@ void ShapeAnnotation::setFileName(QString fileName, Component *pComponent)
     return;
   }
 
-  OMCProxy *pOMCProxy = 0;
-  if (pComponent) {
-    pOMCProxy = pComponent->getGraphicsView()->getModelWidget()->getModelWidgetContainer()->getMainWindow()->getOMCProxy();
-  } else {
-    pOMCProxy = mpGraphicsView->getModelWidget()->getModelWidgetContainer()->getMainWindow()->getOMCProxy();
-  }
-
+  OMCProxy *pOMCProxy = MainWindow::instance()->getOMCProxy();
   mOriginalFileName = fileName;
   QUrl fileUrl(mOriginalFileName);
   QFileInfo fileInfo(mOriginalFileName);
   QFileInfo classFileInfo(mClassFileName);
-
   /* if its a modelica:// link then make it absolute path */
   if (fileUrl.scheme().toLower().compare("modelica") == 0) {
     mFileName = pOMCProxy->uriToFilename(mOriginalFileName);
@@ -758,6 +780,27 @@ QImage ShapeAnnotation::getImage()
   return mImage;
 }
 
+/*!
+  Returns a dynamic value or null if no dynamic value exists
+  */
+QVariant ShapeAnnotation::getDynamicValue(QString name)
+{
+  QVariant dynamicValue; // isNull() per default
+  if (mpParentComponent) {
+    ModelWidget *pModelWidget = mpParentComponent->getGraphicsView()->getModelWidget();
+    if (!pModelWidget->getResultFileName().isEmpty()) {
+      QString fullName = pModelWidget->getResultFileName() + "." + mpParentComponent->getComponentInfo()->getName() + "." + name;
+      MainWindow *pMainWindow = MainWindow::instance();
+      VariablesTreeModel *pVariablesTreeModel = pMainWindow->getVariablesWidget()->getVariablesTreeModel();
+      VariablesTreeItem *pVariablesTreeItem = pVariablesTreeModel->findVariablesTreeItem(fullName, pVariablesTreeModel->getRootVariablesTreeItem());
+      if (pVariablesTreeItem != NULL) {
+        dynamicValue = pVariablesTreeItem->getValue(pVariablesTreeItem->getPreviousUnit(), pVariablesTreeItem->getUnit());
+      }
+    }
+  }
+  return dynamicValue;
+}
+
 /*!
  * \brief ShapeAnnotation::applyRotation
  * Applies the rotation on the shape and sets the shape transformation matrix accordingly.
@@ -877,6 +920,9 @@ void ShapeAnnotation::adjustCornerItemsConnectedIndexes()
 void ShapeAnnotation::removeRedundantPointsGeometriesAndCornerItems()
 {
   for (int i = 0 ; i < mPoints.size() ; i++) {
+    if (mPoints.size() <= 2) {
+      break;
+    }
     if ((i+1 < mPoints.size() && mPoints[i].y() == mPoints[i + 1].y() && i+2 < mPoints.size() && mPoints[i + 1].y() == mPoints[i + 2].y()) ||
         (i+1 < mPoints.size() && mPoints[i].x() == mPoints[i + 1].x() && i+2 < mPoints.size() && mPoints[i + 1].x() == mPoints[i + 2].x())) {
       mPoints.removeAt(i + 1);
@@ -937,6 +983,36 @@ void ShapeAnnotation::updateShape(ShapeAnnotation *pShapeAnnotation)
   Q_UNUSED(pShapeAnnotation);
 }
 
+/*!
+ * \brief ShapeAnnotation::initUpdateVisible
+ * Initialize optional DynamicSelect for the visible status
+ */
+void ShapeAnnotation::initUpdateVisible()
+{
+  if (mpParentComponent) {
+    if (!mDynamicVisible.isEmpty()) {
+      updateVisible();
+      connect(mpParentComponent, SIGNAL(displayTextChanged()), SLOT(updateVisible()), Qt::UniqueConnection);
+    }
+  }
+}
+
+/*!
+ * \brief ShapeAnnotation::updateVisible
+ * DynamicSelect for the visible status
+ */
+void ShapeAnnotation::updateVisible()
+{
+  bool visible = mVisible; // model provided default value
+  if (!mDynamicVisible.isEmpty()) {
+    QVariant dynamicValue = getDynamicValue(mDynamicVisible);
+    if (!dynamicValue.isNull()) {
+      visible = dynamicValue.toBool();
+    }
+  }
+  setVisible(visible);
+}
+
 /*!
  * \brief ShapeAnnotation::manhattanizeShape
  * Slot activated when mpManhattanizeShapeAction triggered signal is raised.\n
@@ -945,6 +1021,9 @@ void ShapeAnnotation::updateShape(ShapeAnnotation *pShapeAnnotation)
  */
 void ShapeAnnotation::manhattanizeShape(bool addToStack)
 {
+  if (mSmooth == StringHandler::SmoothBezier) {
+    return;
+  }
   QString oldAnnotation = getOMCShapeAnnotation();
   int startIndex = -1;
   for (int i = 0 ; i < mPoints.size() ; i++) {
@@ -1035,6 +1114,10 @@ void ShapeAnnotation::referenceShapeChanged()
       updateShape(pShapeAnnotation);
       setPos(mOrigin);
       setRotation(mRotation);
+      if (dynamic_cast<TextAnnotation*>(this)) {
+        TextAnnotation *pTextAnnotation = dynamic_cast<TextAnnotation*>(this);
+        pTextAnnotation->updateTextString();
+      }
       update();
       mpParentComponent->shapeUpdated();
     }
@@ -1057,18 +1140,6 @@ void ShapeAnnotation::referenceShapeDeleted()
   }
 }
 
-/*!
-  Slot activated when Delete option is chosen from context menu of the shape.\n
-  Deletes the connection.
-  */
-void ShapeAnnotation::deleteConnection()
-{
-  LineAnnotation *pLineAnnotation = dynamic_cast<LineAnnotation*>(this);
-  if (pLineAnnotation && pLineAnnotation->getLineType() == LineAnnotation::ConnectionType) {
-    mpGraphicsView->deleteConnection(pLineAnnotation);
-  }
-}
-
 /*!
  * \brief ShapeAnnotation::deleteMe
  * Deletes the shape. Slot activated when Del key is pressed while the shape is selected.\n
@@ -1077,7 +1148,16 @@ void ShapeAnnotation::deleteConnection()
 void ShapeAnnotation::deleteMe()
 {
   // delete the shape
-  mpGraphicsView->deleteShape(this);
+  LineAnnotation *pLineAnnotation = dynamic_cast<LineAnnotation*>(this);
+  if (pLineAnnotation && pLineAnnotation->getLineType() == LineAnnotation::ConnectionType) {
+    mpGraphicsView->deleteConnection(pLineAnnotation);
+  } else if (pLineAnnotation && pLineAnnotation->getLineType() == LineAnnotation::TransitionType) {
+    mpGraphicsView->deleteTransition(pLineAnnotation);
+  } else if (pLineAnnotation && pLineAnnotation->getLineType() == LineAnnotation::InitialStateType) {
+    mpGraphicsView->deleteInitialState(pLineAnnotation);
+  } else {
+    mpGraphicsView->deleteShape(this);
+  }
 }
 
 /*!
@@ -1379,42 +1459,44 @@ void ShapeAnnotation::updateCornerItemPoint(int index, QPointF point)
   if (dynamic_cast<LineAnnotation*>(this)) {
     LineAnnotation *pLineAnnotation = dynamic_cast<LineAnnotation*>(this);
     if (pLineAnnotation->getLineType() == LineAnnotation::ConnectionType) {
-      // if moving the 2nd last point then we need to add more points after it to keep the last point manhattanized with connector
-      int secondLastIndex = mPoints.size() - 2;
-      if (index == secondLastIndex) {
-        // just check if additional points are really needed or not.
-        if ((mGeometries[secondLastIndex] == ShapeAnnotation::HorizontalLine && mPoints[index].y() != point.y()) ||
-            (mGeometries[secondLastIndex] == ShapeAnnotation::VerticalLine && mPoints[index].x() != point.x())) {
-          insertPointsGeometriesAndCornerItems(mPoints.size() - 1);
+      if (mPoints.size() > index) {
+        // if moving the 2nd last point then we need to add more points after it to keep the last point manhattanized with connector
+        int secondLastIndex = mPoints.size() - 2;
+        if (index == secondLastIndex) {
+          // just check if additional points are really needed or not.
+          if ((mGeometries[secondLastIndex] == ShapeAnnotation::HorizontalLine && mPoints[index].y() != point.y()) ||
+              (mGeometries[secondLastIndex] == ShapeAnnotation::VerticalLine && mPoints[index].x() != point.x())) {
+            insertPointsGeometriesAndCornerItems(mPoints.size() - 1);
+          }
         }
-      }
-      // if moving the 2nd point then we need to add more points behind it to keep the first point manhattanized with connector
-      if (index == 1) {
-        // just check if additional points are really needed or not.
-        if ((mGeometries[0] == ShapeAnnotation::HorizontalLine && mPoints[index].y() != point.y()) ||
-            (mGeometries[0] == ShapeAnnotation::VerticalLine && mPoints[index].x() != point.x())) {
-          insertPointsGeometriesAndCornerItems(1);
-          index = index + 2;
+        // if moving the 2nd point then we need to add more points behind it to keep the first point manhattanized with connector
+        if (index == 1) {
+          // just check if additional points are really needed or not.
+          if ((mGeometries[0] == ShapeAnnotation::HorizontalLine && mPoints[index].y() != point.y()) ||
+              (mGeometries[0] == ShapeAnnotation::VerticalLine && mPoints[index].x() != point.x())) {
+            insertPointsGeometriesAndCornerItems(1);
+            index = index + 2;
+          }
+        }
+        qreal dx = point.x() - mPoints[index].x();
+        qreal dy = point.y() - mPoints[index].y();
+        mPoints.replace(index, point);
+        // update previous point
+        if (mGeometries.size() > index - 1 && mGeometries[index - 1] == ShapeAnnotation::HorizontalLine && mPoints.size() > index - 1) {
+          mPoints[index - 1] = QPointF(mPoints[index - 1].x(), mPoints[index - 1].y() +  dy);
+          updateCornerItem(index - 1);
+        } else if (mGeometries.size() > index - 1 && mGeometries[index - 1] == ShapeAnnotation::VerticalLine && mPoints.size() > index - 1) {
+          mPoints[index - 1] = QPointF(mPoints[index - 1].x() + dx, mPoints[index - 1].y());
+          updateCornerItem(index - 1);
+        }
+        // update next point
+        if (mGeometries.size() > index && mGeometries[index] == ShapeAnnotation::HorizontalLine && mPoints.size() > index + 1) {
+          mPoints[index + 1] = QPointF(mPoints[index + 1].x(), mPoints[index + 1].y() +  dy);
+          updateCornerItem(index + 1);
+        } else if (mGeometries.size() > index && mGeometries[index] == ShapeAnnotation::VerticalLine && mPoints.size() > index + 1) {
+          mPoints[index + 1] = QPointF(mPoints[index + 1].x() + dx, mPoints[index + 1].y());
+          updateCornerItem(index + 1);
         }
-      }
-      qreal dx = point.x() - mPoints[index].x();
-      qreal dy = point.y() - mPoints[index].y();
-      mPoints.replace(index, point);
-      // update previous point
-      if (mGeometries[index - 1] == ShapeAnnotation::HorizontalLine) {
-        mPoints[index - 1] = QPointF(mPoints[index - 1].x(), mPoints[index - 1].y() +  dy);
-        updateCornerItem(index - 1);
-      } else if (mGeometries[index - 1] == ShapeAnnotation::VerticalLine) {
-        mPoints[index - 1] = QPointF(mPoints[index - 1].x() + dx, mPoints[index - 1].y());
-        updateCornerItem(index - 1);
-      }
-      // update next point
-      if (mGeometries[index] == ShapeAnnotation::HorizontalLine) {
-        mPoints[index + 1] = QPointF(mPoints[index + 1].x(), mPoints[index + 1].y() +  dy);
-        updateCornerItem(index + 1);
-      } else if (mGeometries[index] == ShapeAnnotation::VerticalLine) {
-        mPoints[index + 1] = QPointF(mPoints[index + 1].x() + dx, mPoints[index + 1].y());
-        updateCornerItem(index + 1);
       }
     } else {
       mPoints.replace(index, point);
@@ -1464,14 +1546,58 @@ bool ShapeAnnotation::isLineStraight(QPointF point1, QPointF point2)
  */
 void ShapeAnnotation::showShapeProperties()
 {
-  if (!mpGraphicsView || mpGraphicsView->getModelWidget()->getLibraryTreeItem()->getLibraryType()== LibraryTreeItem::TLM) {
+  if (!mpGraphicsView || mpGraphicsView->getModelWidget()->getLibraryTreeItem()->getLibraryType() == LibraryTreeItem::CompositeModel) {
     return;
   }
-  MainWindow *pMainWindow = mpGraphicsView->getModelWidget()->getModelWidgetContainer()->getMainWindow();
+  MainWindow *pMainWindow = MainWindow::instance();
   ShapePropertiesDialog *pShapePropertiesDialog = new ShapePropertiesDialog(this, pMainWindow);
   pShapePropertiesDialog->exec();
 }
 
+/*!
+ * \brief ShapeAnnotation::showShapeAttributes
+ * Slot activated when Attributes option is chosen from context menu of the shape.
+ */
+void ShapeAnnotation::showShapeAttributes()
+{
+  if (!mpGraphicsView) {
+    return;
+  }
+  LineAnnotation *pConnectionLineAnnotation = dynamic_cast<LineAnnotation*>(this);
+  CompositeModelConnectionAttributes *pCompositeModelConnectionAttributes;
+  pCompositeModelConnectionAttributes = new CompositeModelConnectionAttributes(mpGraphicsView, pConnectionLineAnnotation, true, MainWindow::instance());
+  pCompositeModelConnectionAttributes->exec();
+}
+
+/*!
+ * \brief ShapeAnnotation::editTransition
+ * Slot activated when edit transition option is chosen from context menu of the transition.
+ */
+void ShapeAnnotation::editTransition()
+{
+  if (!mpGraphicsView) {
+    return;
+  }
+  LineAnnotation *pTransitionLineAnnotation = dynamic_cast<LineAnnotation*>(this);
+  CreateOrEditTransitionDialog *pCreateOrEditTransitionDialog = new CreateOrEditTransitionDialog(mpGraphicsView, pTransitionLineAnnotation,
+                                                                                                 true, MainWindow::instance());
+  pCreateOrEditTransitionDialog->exec();
+}
+
+/*!
+ * \brief ShapeAnnotation::alignInterfaces
+ * Slot activated when Align Interfaces option is chosen from context menu of the shape.
+ */
+void ShapeAnnotation::alignInterfaces()
+{
+  if (!mpGraphicsView) {
+    return;
+  }
+  LineAnnotation *pConnectionLineAnnotation = dynamic_cast<LineAnnotation*>(this);
+  AlignInterfacesDialog *pAlignInterfacesDialog = new AlignInterfacesDialog(mpGraphicsView->getModelWidget(), pConnectionLineAnnotation);
+  pAlignInterfacesDialog->exec();
+}
+
 /*!
  * \brief ShapeAnnotation::contextMenuEvent
  * Reimplementation of contextMenuEvent.\n
@@ -1490,7 +1616,21 @@ void ShapeAnnotation::contextMenuEvent(QGraphicsSceneContextMenuEvent *pEvent)
   }
 
   QMenu menu(mpGraphicsView);
-  if (mpGraphicsView->getModelWidget()->getLibraryTreeItem()->getLibraryType()== LibraryTreeItem::TLM) {
+  if (mpGraphicsView->getModelWidget()->getLibraryTreeItem()->getLibraryType()== LibraryTreeItem::CompositeModel) {
+    menu.addAction(mpShapeAttributesAction);
+
+    //Only show align interfaces action for bidirectional connections
+    LineAnnotation *pConnectionLineAnnotation = dynamic_cast<LineAnnotation*>(this);
+    QString startName = pConnectionLineAnnotation->getStartComponentName();
+    QString endName = pConnectionLineAnnotation->getEndComponentName();
+    CompositeModelEditor *pEditor = dynamic_cast<CompositeModelEditor*>(mpGraphicsView->getModelWidget()->getEditor());
+    if(pEditor->getInterfaceCausality(startName) == StringHandler::getTLMCausality(StringHandler::TLMBidirectional) &&
+       pEditor->getInterfaceCausality(endName) == StringHandler::getTLMCausality(StringHandler::TLMBidirectional)) {
+        menu.addSeparator();
+        menu.addAction(mpAlignInterfacesAction);
+    }
+
+    menu.addSeparator();
     menu.addAction(mpGraphicsView->getDeleteAction());
   } else {
     menu.addAction(mpShapePropertiesAction);
@@ -1510,12 +1650,12 @@ void ShapeAnnotation::contextMenuEvent(QGraphicsSceneContextMenuEvent *pEvent)
     LineAnnotation::LineType lineType = LineAnnotation::ShapeType;
     if (pLineAnnotation) {
       lineType = pLineAnnotation->getLineType();
-      if (lineType != LineAnnotation::ConnectionType) {
+      if (lineType != LineAnnotation::ConnectionType && lineType != LineAnnotation::TransitionType) {
         menu.addAction(mpGraphicsView->getManhattanizeAction());
       }
     }
     menu.addAction(mpGraphicsView->getDeleteAction());
-    if (lineType != LineAnnotation::ConnectionType) {
+    if (lineType != LineAnnotation::ConnectionType && lineType != LineAnnotation::TransitionType) {
       menu.addAction(mpGraphicsView->getDuplicateAction());
       menu.addSeparator();
       menu.addAction(mpGraphicsView->getBringToFrontAction());
@@ -1525,6 +1665,9 @@ void ShapeAnnotation::contextMenuEvent(QGraphicsSceneContextMenuEvent *pEvent)
       menu.addSeparator();
       menu.addAction(mpGraphicsView->getRotateClockwiseAction());
       menu.addAction(mpGraphicsView->getRotateAntiClockwiseAction());
+    } else if (lineType == LineAnnotation::TransitionType) {
+      menu.addSeparator();
+      menu.addAction(mpEditTransitionAction);
     }
   }
   menu.exec(pEvent->screenPos());
@@ -1551,14 +1694,12 @@ QVariant ShapeAnnotation::itemChange(GraphicsItemChange change, const QVariant &
       setCursor(Qt::SizeAllCursor);
       /* Only allow manipulations on shapes if the class is not a system library class OR shape is not an inherited component. */
       if (!mpGraphicsView->getModelWidget()->getLibraryTreeItem()->isSystemLibrary() && !isInheritedShape()) {
-        if (lineType == LineAnnotation::ConnectionType) {
-          connect(mpGraphicsView, SIGNAL(mouseDelete()), SLOT(deleteConnection()), Qt::UniqueConnection);
-          connect(mpGraphicsView, SIGNAL(keyPressDelete()), SLOT(deleteConnection()), Qt::UniqueConnection);
-        } else {
-          if (pLineAnnotation) {
-            connect(mpGraphicsView, SIGNAL(mouseManhattanize()), this, SLOT(manhattanizeShape()), Qt::UniqueConnection);
-          }
-          connect(mpGraphicsView, SIGNAL(mouseDelete()), this, SLOT(deleteMe()), Qt::UniqueConnection);
+        if (pLineAnnotation) {
+          connect(mpGraphicsView, SIGNAL(mouseManhattanize()), this, SLOT(manhattanizeShape()), Qt::UniqueConnection);
+        }
+        connect(mpGraphicsView, SIGNAL(mouseDelete()), this, SLOT(deleteMe()), Qt::UniqueConnection);
+        connect(mpGraphicsView, SIGNAL(keyPressDelete()), this, SLOT(deleteMe()), Qt::UniqueConnection);
+        if (lineType == LineAnnotation::ShapeType) {
           connect(mpGraphicsView, SIGNAL(mouseDuplicate()), this, SLOT(duplicate()), Qt::UniqueConnection);
           connect(mpGraphicsView->getBringToFrontAction(), SIGNAL(triggered()), this, SLOT(bringToFront()), Qt::UniqueConnection);
           connect(mpGraphicsView->getBringForwardAction(), SIGNAL(triggered()), this, SLOT(bringForward()), Qt::UniqueConnection);
@@ -1566,7 +1707,6 @@ QVariant ShapeAnnotation::itemChange(GraphicsItemChange change, const QVariant &
           connect(mpGraphicsView->getSendBackwardAction(), SIGNAL(triggered()), this, SLOT(sendBackward()), Qt::UniqueConnection);
           connect(mpGraphicsView, SIGNAL(mouseRotateClockwise()), this, SLOT(rotateClockwise()), Qt::UniqueConnection);
           connect(mpGraphicsView, SIGNAL(mouseRotateAntiClockwise()), this, SLOT(rotateAntiClockwise()), Qt::UniqueConnection);
-          connect(mpGraphicsView, SIGNAL(keyPressDelete()), this, SLOT(deleteMe()), Qt::UniqueConnection);
           connect(mpGraphicsView, SIGNAL(keyPressDuplicate()), this, SLOT(duplicate()), Qt::UniqueConnection);
           connect(mpGraphicsView, SIGNAL(keyPressRotateClockwise()), this, SLOT(rotateClockwise()), Qt::UniqueConnection);
           connect(mpGraphicsView, SIGNAL(keyPressRotateAntiClockwise()), this, SLOT(rotateAntiClockwise()), Qt::UniqueConnection);
@@ -1589,14 +1729,12 @@ QVariant ShapeAnnotation::itemChange(GraphicsItemChange change, const QVariant &
       unsetCursor();
       /* Only allow manipulations on shapes if the class is not a system library class OR shape is not an inherited component. */
       if (!mpGraphicsView->getModelWidget()->getLibraryTreeItem()->isSystemLibrary() && !isInheritedShape()) {
-        if (lineType == LineAnnotation::ConnectionType) {
-          disconnect(mpGraphicsView, SIGNAL(mouseDelete()), this, SLOT(deleteConnection()));
-          disconnect(mpGraphicsView, SIGNAL(keyPressDelete()), this, SLOT(deleteConnection()));
-        } else {
-          if (pLineAnnotation) {
-            disconnect(mpGraphicsView, SIGNAL(mouseManhattanize()), this, SLOT(manhattanizeShape()));
-          }
-          disconnect(mpGraphicsView, SIGNAL(mouseDelete()), this, SLOT(deleteMe()));
+        if (pLineAnnotation) {
+          disconnect(mpGraphicsView, SIGNAL(mouseManhattanize()), this, SLOT(manhattanizeShape()));
+        }
+        disconnect(mpGraphicsView, SIGNAL(mouseDelete()), this, SLOT(deleteMe()));
+        disconnect(mpGraphicsView, SIGNAL(keyPressDelete()), this, SLOT(deleteMe()));
+        if (lineType == LineAnnotation::ShapeType) {
           disconnect(mpGraphicsView, SIGNAL(mouseDuplicate()), this, SLOT(duplicate()));
           disconnect(mpGraphicsView->getBringToFrontAction(), SIGNAL(triggered()), this, SLOT(bringToFront()));
           disconnect(mpGraphicsView->getBringForwardAction(), SIGNAL(triggered()), this, SLOT(bringForward()));
@@ -1604,7 +1742,6 @@ QVariant ShapeAnnotation::itemChange(GraphicsItemChange change, const QVariant &
           disconnect(mpGraphicsView->getSendBackwardAction(), SIGNAL(triggered()), this, SLOT(sendBackward()));
           disconnect(mpGraphicsView, SIGNAL(mouseRotateClockwise()), this, SLOT(rotateClockwise()));
           disconnect(mpGraphicsView, SIGNAL(mouseRotateAntiClockwise()), this, SLOT(rotateAntiClockwise()));
-          disconnect(mpGraphicsView, SIGNAL(keyPressDelete()), this, SLOT(deleteMe()));
           disconnect(mpGraphicsView, SIGNAL(keyPressDuplicate()), this, SLOT(duplicate()));
           disconnect(mpGraphicsView, SIGNAL(keyPressRotateClockwise()), this, SLOT(rotateClockwise()));
           disconnect(mpGraphicsView, SIGNAL(keyPressRotateAntiClockwise()), this, SLOT(rotateAntiClockwise()));
