@@ -29,20 +29,23 @@
  *
  */
 /*
- *
  * @author Adeel Asghar <adeel.asghar@liu.se>
- *
- * RCS: $Id$
- *
  */
 
 #include <limits>
 
 #include "ShapePropertiesDialog.h"
-#include "Commands.h"
+#include "MainWindow.h"
+#include "Options/OptionsDialog.h"
+#include "Options/NotificationsDialog.h"
+#include "Modeling/Commands.h"
 
-ShapePropertiesDialog::ShapePropertiesDialog(ShapeAnnotation *pShapeAnnotation, MainWindow *pMainWindow)
-  : QDialog(pMainWindow, Qt::WindowTitleHint)
+#include <QHeaderView>
+#include <QColorDialog>
+#include <QMessageBox>
+
+ShapePropertiesDialog::ShapePropertiesDialog(ShapeAnnotation *pShapeAnnotation, QWidget *pParent)
+  : QDialog(pParent)
 {
   mpShapeAnnotation = pShapeAnnotation;
   mOldAnnotation = "";
@@ -52,18 +55,13 @@ ShapePropertiesDialog::ShapePropertiesDialog(ShapeAnnotation *pShapeAnnotation,
   mpEllipseAnnotation = dynamic_cast<EllipseAnnotation*>(mpShapeAnnotation);
   mpTextAnnotation = dynamic_cast<TextAnnotation*>(mpShapeAnnotation);
   mpBitmapAnnotation = dynamic_cast<BitmapAnnotation*>(mpShapeAnnotation);
-  mpMainWindow = pMainWindow;
   QString title = getTitle();
   setWindowTitle(QString(Helper::applicationName).append(" - ").append(title).append(" ").append(Helper::properties));
   setAttribute(Qt::WA_DeleteOnClose);
   // heading label
-  mpShapePropertiesHeading = new Label(QString(title).append(" ").append(Helper::properties));
-  mpShapePropertiesHeading->setFont(QFont(Helper::systemFontInfo.family(), Helper::headingFontSize));
-  mpShapePropertiesHeading->setAlignment(Qt::AlignTop);
+  mpShapePropertiesHeading = Utilities::getHeadingLabel(QString(title).append(" ").append(Helper::properties));
   // set separator line
-  mHorizontalLine = new QFrame();
-  mHorizontalLine->setFrameShape(QFrame::HLine);
-  mHorizontalLine->setFrameShadow(QFrame::Sunken);
+  mHorizontalLine = Utilities::getHeadingLine();
   // Transformations Group Box
   mpTransformationGroupBox = new QGroupBox(tr("Transformation"));
   mpOriginXLabel = new Label(Helper::originX);
@@ -185,6 +183,7 @@ ShapePropertiesDialog::ShapePropertiesDialog(ShapeAnnotation *pShapeAnnotation,
   // Text Group Box
   mpTextGroupBox = new QGroupBox(tr("Text"));
   mpTextTextBox = new QLineEdit(mpShapeAnnotation->getTextString());
+  mpTextTextBox->setToolTip(tr("Use \\n for multi-line text"));
   // set the Text Group Box layout
   QHBoxLayout *pTextGroupBoxLayout = new QHBoxLayout;
   pTextGroupBoxLayout->addWidget(mpTextTextBox);
@@ -206,11 +205,11 @@ ShapePropertiesDialog::ShapePropertiesDialog(ShapeAnnotation *pShapeAnnotation,
   mpFontSizeSpinBox->setValue(mpShapeAnnotation->getFontSize());
   mpFontSizeSpinBox->setSingleStep(1);
   mpFontStyleLabel = new Label(tr("Style:"));
-  mpTextBoldCheckBox = new QCheckBox(tr("Bold"));
+  mpTextBoldCheckBox = new QCheckBox(Helper::bold);
   mpTextBoldCheckBox->setChecked(StringHandler::getFontWeight(mpShapeAnnotation->getTextStyles()) == QFont::Bold ? true : false);
-  mpTextItalicCheckBox = new QCheckBox(tr("Italic"));
+  mpTextItalicCheckBox = new QCheckBox(Helper::italic);
   mpTextItalicCheckBox->setChecked(StringHandler::getFontItalic(mpShapeAnnotation->getTextStyles()));
-  mpTextUnderlineCheckBox = new QCheckBox(tr("Underline"));
+  mpTextUnderlineCheckBox = new QCheckBox(Helper::underline);
   mpTextUnderlineCheckBox->setChecked(StringHandler::getFontUnderline(mpShapeAnnotation->getTextStyles()));
   mpTextHorizontalAlignmentLabel = new Label(tr("Horizontal Alignment:"));
   mpTextHorizontalAlignmentComboBox = new QComboBox;
@@ -337,7 +336,7 @@ ShapePropertiesDialog::ShapePropertiesDialog(ShapeAnnotation *pShapeAnnotation,
   mpFillStyleGroupBox->setLayout(pFillStyleGroupBoxLayout);
   // Image Group Box
   mpImageGroupBox = new QGroupBox(tr("Image"));
-  mpFileLabel = new Label(Helper::file);
+  mpFileLabel = new Label(Helper::fileLabel);
   mpFileTextBox = new QLineEdit(mpShapeAnnotation->getFileName());
   mpFileTextBox->setEnabled(false);
   mpBrowseFileButton = new QPushButton(Helper::browse);
@@ -372,6 +371,7 @@ ShapePropertiesDialog::ShapePropertiesDialog(ShapeAnnotation *pShapeAnnotation,
 #else /* Qt4 */
   mpPointsTableWidget->horizontalHeader()->setResizeMode(QHeaderView::Stretch);
 #endif
+  mpPointsTableWidget->horizontalHeader()->setDefaultAlignment(Qt::AlignLeft);
   QStringList headerLabels;
   headerLabels << "X" << "Y";
   mpPointsTableWidget->setHorizontalHeaderLabels(headerLabels);
@@ -502,6 +502,10 @@ QString ShapePropertiesDialog::getTitle()
   if (mpLineAnnotation) {
     if (mpLineAnnotation->getLineType() == LineAnnotation::ConnectionType) {
       return "Connection";
+    } else if (mpLineAnnotation->getLineType() == LineAnnotation::TransitionType) {
+      return "Transition";
+    } else if (mpLineAnnotation->getLineType() == LineAnnotation::InitialStateType) {
+      return "Initial State";
     } else {
       return "Line";
     }
@@ -580,12 +584,15 @@ void ShapePropertiesDialog::movePointUp()
 {
   LineAnnotation::LineType lineType = LineAnnotation::ShapeType;
   if (mpLineAnnotation) lineType = mpLineAnnotation->getLineType();
+  if (lineType == LineAnnotation::InitialStateType) {
+    return;
+  }
   if (mpPointsTableWidget->selectedItems().size() > 0) {
     int row = mpPointsTableWidget->selectedItems().at(0)->row();
     if (row == 0) {
       return;
     }
-    if (row == 1 && lineType == LineAnnotation::ConnectionType) {
+    if (row == 1 && (lineType == LineAnnotation::ConnectionType || lineType == LineAnnotation::TransitionType)) {
       return;
     }
     QTableWidgetItem *pSourceItemX = mpPointsTableWidget->takeItem(row, 0);
@@ -604,12 +611,15 @@ void ShapePropertiesDialog::movePointDown()
 {
   LineAnnotation::LineType lineType = LineAnnotation::ShapeType;
   if (mpLineAnnotation) lineType = mpLineAnnotation->getLineType();
+  if (lineType == LineAnnotation::InitialStateType) {
+    return;
+  }
   if (mpPointsTableWidget->selectedItems().size() > 0) {
     int row = mpPointsTableWidget->selectedItems().at(0)->row();
     if (row == mpPointsTableWidget->rowCount() - 1) {
       return;
     }
-    if (row == mpPointsTableWidget->rowCount() - 2 && lineType == LineAnnotation::ConnectionType) {
+    if (row == mpPointsTableWidget->rowCount() - 2 && (lineType == LineAnnotation::ConnectionType || lineType == LineAnnotation::TransitionType)) {
       return;
     }
     QTableWidgetItem *pSourceItemX = mpPointsTableWidget->takeItem(row, 0);
@@ -627,6 +637,11 @@ void ShapePropertiesDialog::movePointDown()
 void ShapePropertiesDialog::addPoint()
 {
   if (mpPointsTableWidget->selectedItems().size() > 0) {
+    LineAnnotation::LineType lineType = LineAnnotation::ShapeType;
+    if (mpLineAnnotation) lineType = mpLineAnnotation->getLineType();
+    if (lineType == LineAnnotation::InitialStateType) {
+      return;
+    }
     int row = mpPointsTableWidget->selectedItems().at(0)->row();
     if (row == mpPointsTableWidget->rowCount() - 1) {
       /* insert a new row which is similar to last row. */
@@ -653,22 +668,22 @@ void ShapePropertiesDialog::addPoint()
       mpPointsTableWidget->setItem(row + 1, 1, pTableWidgetItemY);
       mpPointsTableWidget->setCurrentCell(row + 1, 0);
     }
-  } else if (mpLineAnnotation && mpPointsTableWidget->rowCount() == 2) {
-    if (mpLineAnnotation->getLineType() == LineAnnotation::ConnectionType) {
-      /* get middle of two surronding points */
-      QPointF point1(mpPointsTableWidget->item(0, 0)->text().toFloat(), mpPointsTableWidget->item(0, 1)->text().toFloat());
-      QPointF point2(mpPointsTableWidget->item(1, 0)->text().toFloat(), mpPointsTableWidget->item(1, 1)->text().toFloat());
-      QPointF point3 = (point1 + point2) / 2;
-      /* insert new row */
-      mpPointsTableWidget->insertRow(1);
-      QTableWidgetItem *pTableWidgetItemX = new QTableWidgetItem(QString::number(point3.x()));
-      pTableWidgetItemX->setFlags(pTableWidgetItemX->flags() | Qt::ItemIsEditable);
-      mpPointsTableWidget->setItem(1, 0, pTableWidgetItemX);
-      QTableWidgetItem *pTableWidgetItemY = new QTableWidgetItem(QString::number(point3.y()));
-      pTableWidgetItemX->setFlags(pTableWidgetItemX->flags() | Qt::ItemIsEditable);
-      mpPointsTableWidget->setItem(1, 1, pTableWidgetItemY);
-      mpPointsTableWidget->setCurrentCell(1, 0);
-    }
+  } else if (mpLineAnnotation && mpPointsTableWidget->rowCount() == 2 &&
+             (mpLineAnnotation->getLineType() == LineAnnotation::ConnectionType ||
+              mpLineAnnotation->getLineType() == LineAnnotation::TransitionType)) {
+    /* get middle of two surronding points */
+    QPointF point1(mpPointsTableWidget->item(0, 0)->text().toFloat(), mpPointsTableWidget->item(0, 1)->text().toFloat());
+    QPointF point2(mpPointsTableWidget->item(1, 0)->text().toFloat(), mpPointsTableWidget->item(1, 1)->text().toFloat());
+    QPointF point3 = (point1 + point2) / 2;
+    /* insert new row */
+    mpPointsTableWidget->insertRow(1);
+    QTableWidgetItem *pTableWidgetItemX = new QTableWidgetItem(QString::number(point3.x()));
+    pTableWidgetItemX->setFlags(pTableWidgetItemX->flags() | Qt::ItemIsEditable);
+    mpPointsTableWidget->setItem(1, 0, pTableWidgetItemX);
+    QTableWidgetItem *pTableWidgetItemY = new QTableWidgetItem(QString::number(point3.y()));
+    pTableWidgetItemX->setFlags(pTableWidgetItemX->flags() | Qt::ItemIsEditable);
+    mpPointsTableWidget->setItem(1, 1, pTableWidgetItemY);
+    mpPointsTableWidget->setCurrentCell(1, 0);
   }
 }
 
@@ -676,7 +691,13 @@ void ShapePropertiesDialog::removePoint()
 {
   if (mpPointsTableWidget->selectedItems().size() > 0) {
     if ((mpLineAnnotation && mpPointsTableWidget->rowCount() > 2) || (mpPolygonAnnotation && mpPointsTableWidget->rowCount() > 4)) {
-      mpPointsTableWidget->removeRow(mpPointsTableWidget->selectedItems().at(0)->row());
+      LineAnnotation::LineType lineType = LineAnnotation::ShapeType;
+      if (mpLineAnnotation) lineType = mpLineAnnotation->getLineType();
+      int row = mpPointsTableWidget->selectedItems().at(0)->row();
+      if (lineType == LineAnnotation::InitialStateType && row == 0) {
+        return;
+      }
+      mpPointsTableWidget->removeRow(row);
     }
   }
 }
@@ -691,6 +712,8 @@ void ShapePropertiesDialog::saveShapeProperties()
 
 bool ShapePropertiesDialog::applyShapeProperties()
 {
+  // we need to set focus on the OK button otherwise QTableWidget doesn't read any active cell editing value.
+  mpOkButton->setFocus(Qt::ActiveWindowFocusReason);
   // save the old annotation before applying anything.
   mOldAnnotation = mpShapeAnnotation->getOMCShapeAnnotation();
   /* validate points */
@@ -699,7 +722,7 @@ bool ShapePropertiesDialog::applyShapeProperties()
     bool Ok;
     pTableWidgetItem->text().toFloat(&Ok);
     if (!Ok || pTableWidgetItem->text().isEmpty()) {
-      QMessageBox::critical(mpMainWindow, QString(Helper::applicationName).append(" - ").append(Helper::error),
+      QMessageBox::critical(MainWindow::instance(), QString(Helper::applicationName).append(" - ").append(Helper::error),
                             GUIMessages::getMessage(GUIMessages::ENTER_VALID_NUMBER)
                             .arg("points item ("+  QString::number(i+1) +",0)"), Helper::ok);
       mpPointsTableWidget->editItem(pTableWidgetItem);
@@ -708,7 +731,7 @@ bool ShapePropertiesDialog::applyShapeProperties()
     pTableWidgetItem = mpPointsTableWidget->item(i, 1); /* point Y value */
     pTableWidgetItem->text().toFloat(&Ok);
     if (!Ok || pTableWidgetItem->text().isEmpty()) {
-      QMessageBox::critical(mpMainWindow, QString(Helper::applicationName).append(" - ").append(Helper::error),
+      QMessageBox::critical(MainWindow::instance(), QString(Helper::applicationName).append(" - ").append(Helper::error),
                             GUIMessages::getMessage(GUIMessages::ENTER_VALID_NUMBER)
                             .arg("points table ["+  QString::number(i+1) +",1]"), Helper::ok);
       mpPointsTableWidget->editItem(pTableWidgetItem);
@@ -719,15 +742,15 @@ bool ShapePropertiesDialog::applyShapeProperties()
   if (mpBitmapAnnotation) {
     if (mpStoreImageInModelCheckBox->isChecked() && mpShapeAnnotation->getImageSource().isEmpty()) {
       if (mpFileTextBox->text().isEmpty()) {
-        QMessageBox::critical(mpMainWindow, QString(Helper::applicationName).append(" - ").append(Helper::error),
-                              GUIMessages::getMessage(GUIMessages::ENTER_NAME).arg(Helper::file), Helper::ok);
+        QMessageBox::critical(MainWindow::instance(), QString(Helper::applicationName).append(" - ").append(Helper::error),
+                              GUIMessages::getMessage(GUIMessages::ENTER_NAME).arg(Helper::fileLabel), Helper::ok);
         mpFileTextBox->setFocus();
         return false;
       }
     } else if (!mpStoreImageInModelCheckBox->isChecked()) {
       if (mpFileTextBox->text().isEmpty()) {
-        QMessageBox::critical(mpMainWindow, QString(Helper::applicationName).append(" - ").append(Helper::error),
-                              GUIMessages::getMessage(GUIMessages::ENTER_NAME).arg(Helper::file), Helper::ok);
+        QMessageBox::critical(MainWindow::instance(), QString(Helper::applicationName).append(" - ").append(Helper::error),
+                              GUIMessages::getMessage(GUIMessages::ENTER_NAME).arg(Helper::fileLabel), Helper::ok);
         mpFileTextBox->setFocus();
         return false;
       }
@@ -791,7 +814,7 @@ bool ShapePropertiesDialog::applyShapeProperties()
         QUrl fileUrl(mpFileTextBox->text());
         QFileInfo fileInfo(mpFileTextBox->text());
         QFileInfo classFileInfo(mpShapeAnnotation->getGraphicsView()->getModelWidget()->getLibraryTreeItem()->getFileName());
-        MainWindow *pMainWindow = mpShapeAnnotation->getGraphicsView()->getModelWidget()->getModelWidgetContainer()->getMainWindow();
+        MainWindow *pMainWindow = MainWindow::instance();
         /* if its a modelica:// link then make it absolute path */
         QString fileName;
         if (fileUrl.scheme().toLower().compare("modelica") == 0) {
@@ -811,7 +834,7 @@ bool ShapePropertiesDialog::applyShapeProperties()
       mpShapeAnnotation->setImage(mpPreviewImageLabel->pixmap()->toImage());
     } else {
       /* find the class to create a relative path */
-      MainWindow *pMainWindow = mpShapeAnnotation->getGraphicsView()->getModelWidget()->getModelWidgetContainer()->getMainWindow();
+      MainWindow *pMainWindow = MainWindow::instance();
       LibraryTreeItem *pLibraryTreeItem = mpShapeAnnotation->getGraphicsView()->getModelWidget()->getLibraryTreeItem();
       QString nameStructure = StringHandler::getFirstWordBeforeDot(pLibraryTreeItem->getNameStructure());
       pLibraryTreeItem = pMainWindow->getLibraryWidget()->getLibraryTreeModel()->findLibraryTreeItem(nameStructure);
@@ -819,8 +842,7 @@ bool ShapePropertiesDialog::applyShapeProperties()
       QFileInfo classFileInfo(pLibraryTreeItem->getFileName());
       QDir classDirectory = classFileInfo.absoluteDir();
       QString relativeImagePath = classDirectory.relativeFilePath(mpFileTextBox->text());
-      mpShapeAnnotation->setFileName(QString("modelica://").append(pLibraryTreeItem->getNameStructure()).append("/")
-                                     .append(relativeImagePath));
+      mpShapeAnnotation->setFileName(QString("modelica://").append(pLibraryTreeItem->getNameStructure()).append("/").append(relativeImagePath));
       mpShapeAnnotation->setImageSource("");
       mpShapeAnnotation->setImage(mpPreviewImageLabel->pixmap()->toImage());
     }
@@ -828,6 +850,9 @@ bool ShapePropertiesDialog::applyShapeProperties()
   LineAnnotation::LineType lineType = LineAnnotation::ShapeType;
   if (mpLineAnnotation) {
     lineType = mpLineAnnotation->getLineType();
+    if (lineType == LineAnnotation::ConnectionType || lineType == LineAnnotation::TransitionType) {
+      mpLineAnnotation->adjustGeometries();
+    }
   }
   // if nothing has changed then just simply return true.
   if (mOldAnnotation.compare(mpShapeAnnotation->getOMCShapeAnnotation()) == 0) {
@@ -837,14 +862,31 @@ bool ShapePropertiesDialog::applyShapeProperties()
     UpdateConnectionCommand *pUpdateConnectionCommand;
     pUpdateConnectionCommand = new UpdateConnectionCommand(mpLineAnnotation, mOldAnnotation, mpShapeAnnotation->getOMCShapeAnnotation());
     mpShapeAnnotation->getGraphicsView()->getModelWidget()->getUndoStack()->push(pUpdateConnectionCommand);
-    mpShapeAnnotation->getGraphicsView()->getModelWidget()->updateModelicaText();
+    mpShapeAnnotation->getGraphicsView()->getModelWidget()->updateModelText();
+  } else if (mpLineAnnotation && lineType == LineAnnotation::TransitionType) {
+    // create a UpdateTransitionCommand object and push it to the undo stack.
+    UpdateTransitionCommand *pUpdateTransitionCommand;
+    pUpdateTransitionCommand = new UpdateTransitionCommand(mpLineAnnotation, mpLineAnnotation->getCondition(), mpLineAnnotation->getImmediate(),
+                                                           mpLineAnnotation->getReset(), mpLineAnnotation->getSynchronize(),
+                                                           mpLineAnnotation->getPriority(), mOldAnnotation, mpLineAnnotation->getCondition(),
+                                                           mpLineAnnotation->getImmediate(), mpLineAnnotation->getReset(),
+                                                           mpLineAnnotation->getSynchronize(), mpLineAnnotation->getPriority(),
+                                                           mpShapeAnnotation->getOMCShapeAnnotation());
+    mpShapeAnnotation->getGraphicsView()->getModelWidget()->getUndoStack()->push(pUpdateTransitionCommand);
+    mpShapeAnnotation->getGraphicsView()->getModelWidget()->updateModelText();
+  } else if (mpLineAnnotation && lineType == LineAnnotation::InitialStateType) {
+    // create a UpdateInitialStateCommand object and push it to the undo stack.
+    UpdateInitialStateCommand *pUpdateInitialStateCommand;
+    pUpdateInitialStateCommand = new UpdateInitialStateCommand(mpLineAnnotation, mOldAnnotation, mpShapeAnnotation->getOMCShapeAnnotation());
+    mpShapeAnnotation->getGraphicsView()->getModelWidget()->getUndoStack()->push(pUpdateInitialStateCommand);
+    mpShapeAnnotation->getGraphicsView()->getModelWidget()->updateModelText();
   } else {
     // create a UpdateShapeCommand object and push it to the undo stack.
     UpdateShapeCommand *pUpdateShapeCommand;
     pUpdateShapeCommand = new UpdateShapeCommand(mpShapeAnnotation, mOldAnnotation, mpShapeAnnotation->getOMCShapeAnnotation());
     mpShapeAnnotation->getGraphicsView()->getModelWidget()->getUndoStack()->push(pUpdateShapeCommand);
     mpShapeAnnotation->getGraphicsView()->getModelWidget()->updateClassAnnotationIfNeeded();
-    mpShapeAnnotation->getGraphicsView()->getModelWidget()->updateModelicaText();
+    mpShapeAnnotation->getGraphicsView()->getModelWidget()->updateModelText();
   }
   return true;
 }
@@ -868,9 +910,9 @@ void ShapePropertiesDialog::storeImageInModelToggled(bool checked)
    * If the model is not saved then make the check box checked again.
    */
   if (!checked) {
-    MainWindow *pMainWindow = mpBitmapAnnotation->getGraphicsView()->getModelWidget()->getModelWidgetContainer()->getMainWindow();
+    MainWindow *pMainWindow = MainWindow::instance();
     if (!mpBitmapAnnotation->getGraphicsView()->getModelWidget()->getLibraryTreeItem()->isFilePathValid()) {
-      if (pMainWindow->getOptionsDialog()->getNotificationsPage()->getSaveModelForBitmapInsertionCheckBox()->isChecked()) {
+      if (OptionsDialog::instance()->getNotificationsPage()->getSaveModelForBitmapInsertionCheckBox()->isChecked()) {
         NotificationsDialog *pNotificationsDialog = new NotificationsDialog(NotificationsDialog::SaveModelForBitmapInsertion,
                                                                             NotificationsDialog::InformationIcon,
                                                                             pMainWindow);
