@@ -1158,9 +1158,15 @@ void GDBAdapter::handleGDBMIConsoleStream(GDBMIStreamRecord *pGDBMIStreamRecord)
  */
 void GDBAdapter::handleGDBMILogStream(GDBMIStreamRecord *pGDBMIStreamRecord)
 {
-  QString LogData = StringHandler::unparse(pGDBMIStreamRecord->value.c_str());
-  mPendingLogStreamOutput += LogData;
-  MainWindow::instance()->getTargetOutputWidget()->logDebuggerErrorOutput(LogData);
+  QString logData = StringHandler::unparse(pGDBMIStreamRecord->value.c_str());
+  mPendingLogStreamOutput += logData;
+  /*! \note Skip the log messages we get as a result of pending breakpoint.
+   * e.g., No source file named Catch.omc.
+   */
+  if (logData.startsWith("No source file named")) {
+    return;
+  }
+  MainWindow::instance()->getTargetOutputWidget()->logDebuggerErrorOutput(logData);
 }
 
 /*!
@@ -1484,22 +1490,12 @@ void GDBAdapter::readGDBErrorOutput()
  */
 void GDBAdapter::handleGDBProcessError(QProcess::ProcessError error)
 {
+  Q_UNUSED(error);
   /* this signal is raised when we kill the timed out GDB forcefully. */
   if (isGDBKilled()) {
     return;
   }
-  QString errorString;
-  switch (error) {
-    case QProcess::FailedToStart:
-      errorString = tr("%1 GDB arguments are \"%2\"").arg(mpGDBProcess->errorString()).arg(mGDBArguments.join(" "));
-      break;
-    case QProcess::Crashed:
-      errorString = tr("GDB crashed with the error %1. GDB arguments are \"%2\"").arg(mpGDBProcess->errorString()).arg(mGDBArguments.join(" "));
-      break;
-    default:
-      errorString = tr("Following error has occurred %1. GDB arguments are \"%2\"").arg(mpGDBProcess->errorString()).arg(mGDBArguments.join(" "));
-      break;
-  }
+  QString errorString = GUIMessages::getMessage(GUIMessages::GDB_ERROR).arg(mpGDBProcess->errorString()).arg(mGDBArguments.join(" "));
   MainWindow::instance()->getTargetOutputWidget()->logDebuggerErrorOutput(errorString);
   setGDBRunning(false);
 }
