@@ -38,7 +38,7 @@
 #include "Options/OptionsDialog.h"
 #include "Debugger/Breakpoints/BreakpointMarker.h"
 #include "Util/Helper.h"
-
+#include <QCompleter>
 #include <QMenu>
 #include <QMessageBox>
 
@@ -51,13 +51,53 @@
  * \param pParent
  */
 ModelicaEditor::ModelicaEditor(QWidget *pParent)
-  : BaseEditor(pParent), mLastValidText(""), mTextChanged(false), mForceSetPlainText(false)
+  : BaseEditor(pParent), mLastValidText(""), mTextChanged(false)
 {
-  setCanHaveBreakpoints(true);
+  mpPlainTextEdit->setCanHaveBreakpoints(true);
   /* set the document marker */
   mpDocumentMarker = new DocumentMarker(mpPlainTextEdit->document());
+  QStringList keywords = ModelicaHighlighter::getKeywords();
+  mpPlainTextEdit->insertCompleterKeywords(keywords);
+  QStringList types = ModelicaHighlighter::getTypes();
+  mpPlainTextEdit->insertCompleterTypes(types);
+  QList<CompleterItem> codesnippets = getCodeSnippets();
+  mpPlainTextEdit->insertCompleterCodeSnippets(codesnippets);
 }
 
+/*!
+ * \brief ModelicaEditor::popUpCompleter()
+ * show the popup for keywords and type for autocompletion
+ */
+void ModelicaEditor::popUpCompleter()
+{
+  QCompleter *completer = mpPlainTextEdit->completer();
+  QRect cr = mpPlainTextEdit->cursorRect();
+  cr.setWidth(completer->popup()->sizeHintForColumn(0)+ completer->popup()->verticalScrollBar()->sizeHint().width());
+  completer->complete(cr);
+}
+
+/*!
+ * \brief ModelicaEditor::getCodeSnippets()
+ * returns the list of CompleterItems to the autocompleter
+ */
+QList<CompleterItem> ModelicaEditor::getCodeSnippets()
+{
+  QList<CompleterItem> codesnippetslist;
+  codesnippetslist << CompleterItem("function" ,"function name\n  \nend name;", "name")
+                   << CompleterItem("block" ,"block name\n  \nend name;", "name")
+                   << CompleterItem("model" ,"model name\n  \nend name;", "name")
+                   << CompleterItem("class" ,"class name\n  \nend name;", "name")
+                   << CompleterItem("connector" ,"connector name\n  \nend name;", "name")
+                   << CompleterItem("package" ,"package name\n  \nend name;", "name")
+                   << CompleterItem("record" ,"record name\n  \nend name;", "name")
+                   << CompleterItem("while" ,"while condition loop\n  \nend while;", "condition")
+                   << CompleterItem("if" ,"if condition then\n  \nend if;", "condition")
+                   << CompleterItem("if" ,"if condition then\n  \nelseif condition then\n  \nelse\n  \nend if;", "condition")
+                   << CompleterItem("for" ,"for condition loop\n  \nend for;", "condition")
+                   << CompleterItem("when", "when condition then\n  \nend when;", "condition")
+                   << CompleterItem("when", "when condition then\n  \nelsewhen condition then\n  \nend when;", "condition");
+  return codesnippetslist;
+}
 /*!
  * \brief ModelicaEditor::getClassNames
  * Uses the OMC parseString API to check the class names inside the Modelica Text
@@ -138,8 +178,9 @@ bool ModelicaEditor::validateText(LibraryTreeItem **pLibraryTreeItem)
       pMessageBox->setText(GUIMessages::getMessage(GUIMessages::ERROR_IN_TEXT).arg("Modelica")
                            .append(GUIMessages::getMessage(GUIMessages::CHECK_MESSAGES_BROWSER))
                            .append(GUIMessages::getMessage(GUIMessages::REVERT_PREVIOUS_OR_FIX_ERRORS_MANUALLY)));
-      pMessageBox->addButton(Helper::fixErrorsManually, QMessageBox::AcceptRole);
+      QPushButton *pAcceptRoleButton = pMessageBox->addButton(Helper::fixErrorsManually, QMessageBox::AcceptRole);
       pMessageBox->addButton(Helper::revertToLastCorrectVersion, QMessageBox::RejectRole);
+      pMessageBox->setEscapeButton(pAcceptRoleButton);
       // we set focus to this widget here so when the error dialog is closed Qt gives back the focus to this widget.
       mpPlainTextEdit->setFocus(Qt::ActiveWindowFocusReason);
       int answer = pMessageBox->exec();
@@ -263,8 +304,9 @@ void ModelicaEditor::showContextMenu(QPoint point)
  * Reimplementation of QPlainTextEdit::setPlainText method.
  * Makes sure we dont update if the passed text is same.
  * \param text the string to set.
+ * \param useInserText
  */
-void ModelicaEditor::setPlainText(const QString &text)
+void ModelicaEditor::setPlainText(const QString &text, bool useInserText)
 {
   QMap<int, int> leadingSpacesMap;
   QString contents = text;
@@ -276,16 +318,27 @@ void ModelicaEditor::setPlainText(const QString &text)
   // Only set the text when it is really new
   if (contents != mpPlainTextEdit->toPlainText()) {
     mForceSetPlainText = true;
-    if (mpModelWidget->getLibraryTreeItem()->isInPackageOneFile()) {
+    if (!useInserText) {
       mpPlainTextEdit->setPlainText(contents);
-      storeLeadingSpaces(leadingSpacesMap);
     } else {
-      mpPlainTextEdit->setPlainText(contents);
+      QTextCursor textCursor (mpPlainTextEdit->document());
+      textCursor.beginEditBlock();
+      textCursor.select(QTextCursor::Document);
+      textCursor.insertText(contents);
+      textCursor.endEditBlock();
+      mpPlainTextEdit->setTextCursor(textCursor);
+    }
+    if (mpModelWidget->getLibraryTreeItem()->isInPackageOneFile()) {
+      storeLeadingSpaces(leadingSpacesMap);
     }
     setTextChanged(false);
     mForceSetPlainText = false;
     mLastValidText = contents;
-    foldAll();
+    /* ticket:4409 Object moving in block diagram unfolds all annotations in text view.
+     * Make sure ModelicaHighlighter::highlightBlock is called before calling foldAll.
+     */
+    OptionsDialog::instance()->emitModelicaEditorSettingsChanged();
+    mpPlainTextEdit->foldAll();
   }
 }
 
@@ -382,93 +435,107 @@ void ModelicaHighlighter::initializeSettings()
   rule.mFormat = mFunctionFormat;
   mHighlightingRules.append(rule);
   // keywords
-  QStringList keywordPatterns;
-  keywordPatterns << "\\balgorithm\\b"
-                  << "\\band\\b"
-                  << "\\bannotation\\b"
-                  << "\\bassert\\b"
-                  << "\\bblock\\b"
-                  << "\\bbreak\\b"
-                  << "\\bBoolean\\b"
-                  << "\\bclass\\b"
-                  << "\\bconnect\\b"
-                  << "\\bconnector\\b"
-                  << "\\bconstant\\b"
-                  << "\\bconstrainedby\\b"
-                  << "\\bder\\b"
-                  << "\\bdiscrete\\b"
-                  << "\\beach\\b"
-                  << "\\belse\\b"
-                  << "\\belseif\\b"
-                  << "\\belsewhen\\b"
-                  << "\\bencapsulated\\b"
-                  << "\\bend\\b"
-                  << "\\benumeration\\b"
-                  << "\\bequation\\b"
-                  << "\\bexpandable\\b"
-                  << "\\bextends\\b"
-                  << "\\bexternal\\b"
-                  << "\\bfalse\\b"
-                  << "\\bfinal\\b"
-                  << "\\bflow\\b"
-                  << "\\bfor\\b"
-                  << "\\bfunction\\b"
-                  << "\\bif\\b"
-                  << "\\bimport\\b"
-                  << "\\bimpure\\b"
-                  << "\\bin\\b"
-                  << "\\binitial\\b"
-                  << "\\binner\\b"
-                  << "\\binput\\b"
-                  << "\\bloop\\b"
-                  << "\\bmodel\\b"
-                  << "\\bnot\\b"
-                  << "\\boperator\\b"
-                  << "\\bor\\b"
-                  << "\\bouter\\b"
-                  << "\\boutput\\b"
-                  << "\\boptimization\\b"
-                  << "\\bpackage\\b"
-                  << "\\bparameter\\b"
-                  << "\\bpartial\\b"
-                  << "\\bprotected\\b"
-                  << "\\bpublic\\b"
-                  << "\\bpure\\b"
-                  << "\\brecord\\b"
-                  << "\\bredeclare\\b"
-                  << "\\breplaceable\\b"
-                  << "\\breturn\\b"
-                  << "\\bstream\\b"
-                  << "\\bthen\\b"
-                  << "\\btrue\\b"
-                  << "\\btype\\b"
-                  << "\\bwhen\\b"
-                  << "\\bwhile\\b"
-                  << "\\bwithin\\b";
+  QStringList keywordPatterns = getKeywords();
   foreach (const QString &pattern, keywordPatterns) {
-    rule.mPattern = QRegExp(pattern);
+    QString newPattern = QString("\\b%1\\b").arg(pattern);
+    rule.mPattern = QRegExp(newPattern);
     rule.mFormat = mKeywordFormat;
     mHighlightingRules.append(rule);
   }
   // Modelica types
-  QStringList typePatterns;
-  typePatterns << "\\bString\\b"
-               << "\\bInteger\\b"
-               << "\\bBoolean\\b"
-               << "\\bReal\\b";
+  QStringList typePatterns = getTypes();
   foreach (const QString &pattern, typePatterns) {
-    rule.mPattern = QRegExp(pattern);
+    QString newPattern = QString("\\b%1\\b").arg(pattern);
+    rule.mPattern = QRegExp(newPattern);
     rule.mFormat = mTypeFormat;
     mHighlightingRules.append(rule);
   }
 }
 
+// Function which returns list of keywords for the highlighter
+QStringList ModelicaHighlighter::getKeywords()
+{
+  QStringList keywordsList;
+  keywordsList   << "algorithm"
+                 << "and"
+                 << "annotation"
+                 << "assert"
+                 << "block"
+                 << "break"
+                 << "class"
+                 << "connect"
+                 << "connector"
+                 << "constant"
+                 << "constrainedby"
+                 << "der"
+                 << "discrete"
+                 << "each"
+                 << "else"
+                 << "elseif"
+                 << "elsewhen"
+                 << "encapsulated"
+                 << "end"
+                 << "enumeration"
+                 << "equation"
+                 << "expandable"
+                 << "extends"
+                 << "external"
+                 << "false"
+                 << "final"
+                 << "flow"
+                 << "for"
+                 << "function"
+                 << "if"
+                 << "import"
+                 << "impure"
+                 << "in"
+                 << "initial"
+                 << "inner"
+                 << "input"
+                 << "loop"
+                 << "model"
+                 << "not"
+                 << "operator"
+                 << "or"
+                 << "outer"
+                 << "output"
+                 << "optimization"
+                 << "package"
+                 << "parameter"
+                 << "partial"
+                 << "protected"
+                 << "public"
+                 << "pure"
+                 << "record"
+                 << "redeclare"
+                 << "replaceable"
+                 << "return"
+                 << "stream"
+                 << "then"
+                 << "true"
+                 << "type"
+                 << "when"
+                 << "while"
+                 << "within";
+  return keywordsList;
+}
+
+// Function which returns list of types for the highlighter
+QStringList ModelicaHighlighter::getTypes()
+{
+  QStringList typesList;
+  typesList << "String"
+            << "Integer"
+            << "Boolean"
+            << "Real";
+  return typesList;
+}
+
 /*!
  * \brief ModelicaTextHighlighter::highlightMultiLine
  * Highlights the multilines text.
  * Quoted text or multiline comments.
  * \param text
- * \param text
  */
 void ModelicaHighlighter::highlightMultiLine(const QString &text)
 {
@@ -537,25 +604,31 @@ void ModelicaHighlighter::highlightMultiLine(const QString &text)
         parentheses.append(Parenthesis(Parenthesis::Closed, text[index], index));
       }
     }
-    if (foldingState) {
+    if (pTextBlockUserData && foldingState) {
       // if no single line comment, no multi line comment and no quotes then check for annotation end
       if (blockState < 1 || blockState > 3) {
         if (text[index] == ';') {
-          if (index == text.length() - 1) { // if we have some text after closing the annotation then we don't want to fold it.
-            if (annotationIndex < 0) { // if we have one line annotation, we don't want to fold it.
-              pTextBlockUserData->setFoldingIndent(1);
+          if (pTextBlockUserData) {
+            QString endText = text.mid(index + 1);
+            /* if we have some text after closing the annotation then we don't want to fold it.
+             * ticket:4310 But if the ending text is just white space then fold it.
+             */
+            if (index == text.length() - 1 || TabSettings::firstNonSpace(endText) == endText.length()) {
+              if (annotationIndex < 0) { // if we have one line annotation, we don't want to fold it.
+                pTextBlockUserData->setFoldingIndent(1);
+              }
+              pTextBlockUserData->setFoldingEndIncluded(true);
+            } else {
+              pTextBlockUserData->setFoldingIndent(0);
             }
-            pTextBlockUserData->setFoldingEndIncluded(true);
-          } else {
-            pTextBlockUserData->setFoldingIndent(0);
           }
           foldingState = false;
-        } else if (annotationIndex < 0) { // if we have one line annotation, we don't want to fold it.
+        } else if (pTextBlockUserData && annotationIndex < 0) { // if we have one line annotation, we don't want to fold it.
           pTextBlockUserData->setFoldingIndent(1);
         }
-      } else if (annotationIndex < 0) { // if we have one line annotation, we don't want to fold it.
+      } else if (pTextBlockUserData && annotationIndex < 0) { // if we have one line annotation, we don't want to fold it.
         pTextBlockUserData->setFoldingIndent(1);
-      } else if (startIndex < annotationIndex) {  // if we have annotation word before quote or comment block is starting then fold.
+      } else if (pTextBlockUserData && startIndex < annotationIndex) {  // if we have annotation word before quote or comment block is starting then fold.
         pTextBlockUserData->setFoldingIndent(1);
       }
     } else {
