@@ -41,7 +41,7 @@
 #include "Util/Helper.h"
 #include "Editors/BaseEditor.h"
 #include "Editors/ModelicaEditor.h"
-#include "Editors/MetaModelEditor.h"
+#include "Editors/CompositeModelEditor.h"
 #include "Editors/CEditor.h"
 #include "Editors/TextEditor.h"
 #include "Editors/MetaModelicaEditor.h"
@@ -87,22 +87,29 @@ private:
   bool mIsCustomScale;
   bool mAddClassAnnotationNeeded;
   bool mIsCreatingConnection;
+  bool mIsCreatingTransition;
   bool mIsCreatingLineShape;
   bool mIsCreatingPolygonShape;
   bool mIsCreatingRectangleShape;
   bool mIsCreatingEllipseShape;
   bool mIsCreatingTextShape;
   bool mIsCreatingBitmapShape;
+  bool mIsPanning;
+  QPoint mLastMouseEventPos;
   Component *mpClickedComponent;
+  Component *mpClickedState;
   bool mIsMovingComponentsAndShapes;
   bool mRenderingLibraryPixmap;
   QList<Component*> mComponentsList;
   QList<LineAnnotation*> mConnectionsList;
+  QList<LineAnnotation*> mTransitionsList;
+  QList<LineAnnotation*> mInitialStatesList;
   QList<ShapeAnnotation*> mShapesList;
   QList<Component*> mInheritedComponentsList;
   QList<LineAnnotation*> mInheritedConnectionsList;
   QList<ShapeAnnotation*> mInheritedShapesList;
   LineAnnotation *mpConnectionLineAnnotation;
+  LineAnnotation *mpTransitionLineAnnotation;
   LineAnnotation *mpLineShapeAnnotation;
   PolygonAnnotation *mpPolygonShapeAnnotation;
   RectangleAnnotation *mpRectangleShapeAnnotation;
@@ -138,6 +145,8 @@ public:
   bool isAddClassAnnotationNeeded() {return mAddClassAnnotationNeeded;}
   void setIsCreatingConnection(bool enable);
   bool isCreatingConnection() {return mIsCreatingConnection;}
+  void setIsCreatingTransition(bool enable);
+  bool isCreatingTransition() {return mIsCreatingTransition;}
   void setIsCreatingLineShape(bool enable);
   bool isCreatingLineShape() {return mIsCreatingLineShape;}
   void setIsCreatingPolygonShape(bool enable);
@@ -150,6 +159,9 @@ public:
   bool isCreatingTextShape() {return mIsCreatingTextShape;}
   void setIsCreatingBitmapShape(bool enable);
   bool isCreatingBitmapShape() {return mIsCreatingBitmapShape;}
+  void setIsPanning(bool enable);
+  bool isPanning() {return mIsPanning;}
+  void setDragModeInternal(bool enable, bool updateCursor = false);
   void setItemsFlags(bool enable);
   void updateUndoRedoActions(bool enable);
   void setIsMovingComponentsAndShapes(bool enable) {mIsMovingComponentsAndShapes = enable;}
@@ -170,8 +182,8 @@ public:
   QAction* getFlipHorizontalAction() {return mpFlipHorizontalAction;}
   QAction* getFlipVerticalAction() {return mpFlipVerticalAction;}
   bool addComponent(QString className, QPointF position);
-  void addComponentToView(QString name, LibraryTreeItem *pLibraryTreeItem, QString transformationString, QPointF position,
-                          QStringList dialogAnnotation, ComponentInfo *pComponentInfo, bool addObject = true, bool openingClass = false);
+  void addComponentToView(QString name, LibraryTreeItem *pLibraryTreeItem, QString annotation, QPointF position,
+                          ComponentInfo *pComponentInfo, bool addObject = true, bool openingClass = false);
   void addComponentToList(Component *pComponent) {mComponentsList.append(pComponent);}
   void addInheritedComponentToList(Component *pComponent) {mInheritedComponentsList.append(pComponent);}
   void addComponentToClass(Component *pComponent);
@@ -193,10 +205,20 @@ public:
   void addInheritedConnectionToList(LineAnnotation *pConnectionLineAnnotation) {mInheritedConnectionsList.append(pConnectionLineAnnotation);}
   void deleteConnectionFromList(LineAnnotation *pConnectionLineAnnotation) {mConnectionsList.removeOne(pConnectionLineAnnotation);}
   void deleteInheritedConnectionFromList(LineAnnotation *pConnectionLineAnnotation) {mInheritedConnectionsList.removeOne(pConnectionLineAnnotation);}
-  void addShapeToList(ShapeAnnotation *pShape) {mShapesList.append(pShape);}
+  QList<LineAnnotation*> getTransitionsList() {return mTransitionsList;}
+  void addTransitionToClass(LineAnnotation *pTransitionLineAnnotation);
+  void deleteTransitionFromClass(LineAnnotation *pTransitionLineAnnotation);
+  void addTransitionToList(LineAnnotation *pTransitionLineAnnotation) {mTransitionsList.append(pTransitionLineAnnotation);}
+  void deleteTransitionFromList(LineAnnotation *pTransitionLineAnnotation) {mTransitionsList.removeOne(pTransitionLineAnnotation);}
+  QList<LineAnnotation*> getInitialStatesList() {return mInitialStatesList;}
+  void addInitialStateToClass(LineAnnotation *pInitialStateLineAnnotation);
+  void deleteInitialStateFromClass(LineAnnotation *pInitialStateLineAnnotation);
+  void addInitialStateToList(LineAnnotation *pInitialStateLineAnnotation) {mInitialStatesList.append(pInitialStateLineAnnotation);}
+  void deleteInitialStateFromList(LineAnnotation *pInitialStateLineAnnotation) {mInitialStatesList.removeOne(pInitialStateLineAnnotation);}
+  void addShapeToList(ShapeAnnotation *pShape, int index = -1);
   void addInheritedShapeToList(ShapeAnnotation *pShape) {mInheritedShapesList.append(pShape);}
   void deleteShape(ShapeAnnotation *pShapeAnnotation);
-  void deleteShapeFromList(ShapeAnnotation *pShape) {mShapesList.removeOne(pShape);}
+  int deleteShapeFromList(ShapeAnnotation *pShape);
   void deleteInheritedShapeFromList(ShapeAnnotation *pShape) {mInheritedShapesList.removeOne(pShape);}
   void reOrderShapes();
   void bringToFront(ShapeAnnotation *pShape);
@@ -224,6 +246,8 @@ private:
   void createActions();
   bool isClassDroppedOnItself(LibraryTreeItem *pLibraryTreeItem);
   bool isAnyItemSelectedAndEditable(int key);
+  Component* connectorComponentAtPosition(QPoint position);
+  Component* stateComponentAtPosition(QPoint position);
 signals:
   void mouseManhattanize();
   void mouseDelete();
@@ -254,6 +278,10 @@ public slots:
   void addConnection(Component *pComponent);
   void removeCurrentConnection();
   void deleteConnection(LineAnnotation *pConnectionLineAnnotation);
+  void addTransition(Component *pComponent);
+  void removeCurrentTransition();
+  void deleteTransition(LineAnnotation *pTransitionLineAnnotation);
+  void deleteInitialState(LineAnnotation *pInitialLineAnnotation);
   void resetZoom();
   void zoomIn();
   void zoomOut();
@@ -284,6 +312,7 @@ protected:
   virtual void contextMenuEvent(QContextMenuEvent *event);
   virtual void resizeEvent(QResizeEvent *event);
   virtual void wheelEvent(QWheelEvent *event);
+  virtual void leaveEvent(QEvent *event);
 };
 
 class WelcomePageWidget : public QWidget
@@ -325,7 +354,7 @@ private slots:
 
 class ModelWidgetContainer;
 class ModelicaHighlighter;
-class MetaModelHighlighter;
+class CompositeModelHighlighter;
 class Label;
 class ModelWidget : public QWidget
 {
@@ -345,7 +374,6 @@ public:
   BaseEditor* getEditor() {return mpEditor;}
   void setModelClassPathLabel(QString path) {mpModelClassPathLabel->setText(path);}
   void setModelFilePathLabel(QString path) {mpModelFilePathLabel->setText(path);}
-  Label* getCursorPositionLabel() {return mpCursorPositionLabel;}
   bool isLoadedWidgetComponents() {return mCreateModelWidgetComponents;}
   void addInheritedClass(LibraryTreeItem *pLibraryTreeItem) {mInheritedClassesList.append(pLibraryTreeItem);}
   void removeInheritedClass(LibraryTreeItem *pLibraryTreeItem) {mInheritedClassesList.removeOne(pLibraryTreeItem);}
@@ -364,18 +392,22 @@ public:
   void loadConnections();
   void createModelWidgetComponents();
   Component* getConnectorComponent(Component *pConnectorComponent, QString connectorName);
+  void clearGraphicsViews();
   void reDrawModelWidget();
   bool validateText(LibraryTreeItem **pLibraryTreeItem);
   bool modelicaEditorTextChanged(LibraryTreeItem **pLibraryTreeItem);
   void updateChildClasses(LibraryTreeItem *pLibraryTreeItem);
   void clearSelection();
   void updateClassAnnotationIfNeeded();
-  void updateModelText();
+  void updateModelText(bool updateText = true);
   void updateModelicaTextManually(QString contents);
   void updateUndoRedoActions();
   void updateDynamicResults(QString resultFileName);
   QString getResultFileName() {return mResultFileName;}
-  void writeVisualXMLFile();
+  bool writeCoSimulationResultFile(QString fileName);
+  bool writeVisualXMLFile(QString fileName, bool canWriteVisualXMLFile = false);
+  void beginMacro(const QString &text);
+  void endMacro();
 private:
   ModelWidgetContainer *mpModelWidgetContainer;
   LibraryTreeItem *mpLibraryTreeItem;
@@ -389,7 +421,6 @@ private:
   Label *mpViewTypeLabel;
   Label *mpModelClassPathLabel;
   Label *mpModelFilePathLabel;
-  Label *mpCursorPositionLabel;
   QToolButton *mpFileLockToolButton;
   GraphicsView *mpDiagramGraphicsView;
   GraphicsScene *mpDiagramGraphicsScene;
@@ -416,16 +447,21 @@ private:
   void getModelIconDiagramShapes(StringHandler::ViewType viewType);
   void drawModelInheritedClassComponents(ModelWidget *pModelWidget, StringHandler::ViewType viewType);
   void removeInheritedClassComponents(StringHandler::ViewType viewType);
+  void removeClassComponents(StringHandler::ViewType viewType);
   void getModelComponents();
   void drawModelIconComponents();
   void drawModelDiagramComponents();
   void drawModelInheritedClassConnections(ModelWidget *pModelWidget);
   void removeInheritedClassConnections();
   void getModelConnections();
-  void detectMultipleDeclarations();
-  QString getMetaModelName();
+  void getModelTransitions();
+  void getModelInitialStates();
   void getMetaModelSubModels();
   void getMetaModelConnections();
+  void detectMultipleDeclarations();
+  QString getCompositeModelName();
+  void getCompositeModelSubModels();
+  void getCompositeModelConnections();
 private slots:
   void showIconView(bool checked);
   void showDiagramView(bool checked);
@@ -433,7 +469,7 @@ private slots:
 public slots:
   void makeFileWritAble();
   void showDocumentationView();
-  bool metaModelEditorTextChanged();
+  bool compositeModelEditorTextChanged();
   void handleCanUndoChanged(bool canUndo);
   void handleCanRedoChanged(bool canRedo);
   void removeDynamicResults(QString resultFileName = "");
@@ -441,6 +477,9 @@ protected:
   virtual void closeEvent(QCloseEvent *event);
 };
 
+
+void addCloseActionsToSubWindowSystemMenu(QMdiSubWindow *pMdiSubWindow);
+
 class ModelWidgetContainer : public QMdiArea
 {
   Q_OBJECT
@@ -457,6 +496,9 @@ public:
   bool isShowGridLines() {return mShowGridLines;}
   bool eventFilter(QObject *object, QEvent *event);
   void changeRecentModelsListSelection(bool moveDown);
+#if !defined(WITHOUT_OSG)
+  void updateThreeDViewer(ModelWidget *pModelWidget);
+#endif
 private:
   StringHandler::ViewType mPreviousViewType;
   bool mShowGridLines;
@@ -466,6 +508,7 @@ private:
 public slots:
   bool openRecentModelWidget(QListWidgetItem *pListWidgetItem);
   void currentModelWidgetChanged(QMdiSubWindow *pSubWindow);
+  void updateThreeDViewer(QMdiSubWindow *pSubWindow);
   void saveModelWidget();
   void saveAsModelWidget();
   void saveTotalModelWidget();
