@@ -74,10 +74,8 @@ private:
   QString mObjectRefFile;
   QList<QString> mCommandsList;
   int mCurrentCommandIndex;
-  QFile mCommunicationLogFile;
-  QTextStream mCommunicationLogFileTextStream;
-  QFile mCommandsMosFile;
-  QTextStream mCommandsLogFileTextStream;
+  FILE *mpCommunicationLogFile;
+  FILE *mpCommandsLogFile;
   double mTotalOMCCallsTime;
   QList<UnitConverion> mUnitConversionList;
   QMap<QString, QList<QString> > mDerivedUnitsMap;
@@ -111,7 +109,7 @@ public:
   void loadSystemLibraries();
   void loadUserLibraries();
   QStringList getClassNames(QString className = QString("AllLoadedClasses"), bool recursive = false, bool qualified = false,
-                            bool sort = false, bool builtin = false, bool showProtected = true);
+                            bool sort = false, bool builtin = false, bool showProtected = true, bool includeConstants = false);
   QStringList searchClassNames(QString searchText, bool findInText = false);
   OMCInterface::getClassInformation_res getClassInformation(QString className);
   bool isPackage(QString className);
@@ -136,8 +134,10 @@ public:
   QString getIconAnnotation(QString className);
   QString getDiagramAnnotation(QString className);
   int getConnectionCount(QString className);
-  QString getNthConnection(QString className, int num);
+  QList<QString> getNthConnection(QString className, int index);
   QString getNthConnectionAnnotation(QString className, int num);
+  QList<QList<QString> > getTransitions(QString className);
+  QList<QList<QString> > getInitialStates(QString className);
   int getInheritanceCount(QString className);
   QString getNthInheritedClass(QString className, int num);
   QList<QString> getInheritedClasses(QString className);
@@ -183,6 +183,16 @@ public:
   bool setComponentDimensions(QString className, QString componentName, QString dimensions);
   bool addConnection(QString from, QString to, QString className, QString annotation);
   bool deleteConnection(QString from, QString to, QString className);
+  bool addTransition(QString className, QString from, QString to, QString condition, bool immediate, bool reset, bool synchronize,
+                     int priority, QString annotation);
+  bool deleteTransition(QString className, QString from, QString to, QString condition, bool immediate, bool reset, bool synchronize,
+                        int priority);
+  bool updateTransition(QString className, QString from, QString to, QString oldCondition, bool oldImmediate, bool oldReset,
+                        bool oldSynchronize, int oldPriority, QString condition, bool immediate, bool reset, bool synchronize, int priority,
+                        QString annotation);
+  bool addInitialState(QString className, QString state, QString annotation);
+  bool deleteInitialState(QString className, QString state);
+  bool updateInitialState(QString className, QString state, QString annotation);
   bool simulate(QString className, QString simualtionParameters);
   bool buildModel(QString className, QString simualtionParameters);
   bool translateModel(QString className, QString simualtionParameters);
@@ -193,8 +203,8 @@ public:
   QString checkAllModelsRecursive(QString className);
   bool isExperiment(QString className);
   OMCInterface::getSimulationOptions_res getSimulationOptions(QString className, double defaultTolerance = 1e-6);
-  bool buildModelFMU(QString className, double version, QString type, QString fileNamePrefix, QList<QString> platforms);
-  bool translateModelXML(QString className);
+  QString buildModelFMU(QString className, double version, QString type, QString fileNamePrefix, QList<QString> platforms, bool includeResources = true);
+  QString translateModelXML(QString className);
   QString importFMU(QString fmuName, QString outputDirectory, int logLevel, bool debugLogging, bool generateInputConnectors, bool generateOutputConnectors);
   QString importFMUModelDescription(QString fmuModelDescriptionName, QString outputDirectory, int logLevel, bool debugLogging, bool generateInputConnectors, bool generateOutputConnectors);
   QString getMatchingAlgorithm();
@@ -212,6 +222,8 @@ public:
   QString getDerivedClassModifierValue(QString className, QString modifierName);
   OMCInterface::convertUnits_res convertUnits(QString from, QString to);
   QList<QString> getDerivedUnits(QString baseUnit);
+  QString getVersionDateAnnotation(QString className);
+  QString getVersionBuildAnnotation(QString className);
   bool getDocumentationClassAnnotation(QString className);
   QString getCommandLineOptionsAnnotation(QString className);
   QList<QString> getAnnotationNamedModifiers(QString className, QString annotation);
@@ -235,6 +247,7 @@ public:
   bool moveClassToTop(QString className);
   bool moveClassToBottom(QString className);
   bool inferBindings(QString className);
+  bool generateVerificationScenarios(QString className);
   QList<QList<QString > > getUses(QString className);
 signals:
   void commandFinished();
