@@ -67,6 +67,10 @@ public:
     Debug,
     OMEditInfo  /* used internally by OMEdit to mark message blue. */
   };
+  enum TLMCausality { TLMBidirectional, TLMInput, TLMOutput };
+  static QString getTLMCausality(int causality);
+  enum TLMDomain { Mechanical, Electric, Hydraulic, Pneumatic, Magnetic, Signal };
+  static QString getTLMDomain(int domain);
   enum SimulationTools {Adams, Beast, Dymola, OpenModelica, Simulink, WolframSystemModeler, Other};
   static QString getSimulationTool(int tool);
   static QString getSimulationToolStartCommand(QString tool, QString simulationToolStartCommand);
@@ -105,6 +109,7 @@ public:
   static QString removeFirstLastCurlBrackets(QString value);
   static QString removeFirstLastBrackets(QString value);
   static QString removeFirstLastQuotes(QString value);
+  static QString removeFirstLastSingleQuotes(QString value);
   static QStringList getStrings(QString value);
   static QStringList getStrings(QString value, char start, char end);
   /* Handles quoted identifiers A.B.'C.D' -> A.B, A.B.C.D -> A.B.C */
@@ -133,12 +138,13 @@ public:
   static QString getExistingDirectory(QWidget* parent = 0, const QString &caption = "", QString * dir = 0);
   static void setLastOpenDirectory(QString lastOpenDirectory);
   static QString getLastOpenDirectory();
-  static QStringList getDialogAnnotation(QString componentAnnotation);
+  static QStringList getAnnotation(QString componentAnnotation, QString annotationName);
   static QString getPlacementAnnotation(QString componentAnnotation);
   static qreal getNormalizedAngle(qreal angle);
   static QStringList splitStringWithSpaces(QString value);
   static void fillEncodingComboBox(QComboBox *pEncodingComboBox);
   static QStringList makeVariableParts(QString variable);
+  static QStringList makeVariablePartsWithInd(QString variable);
   static bool naturalSort(const QString &s1, const QString &s2);
 #ifdef WIN32
   static QProcessEnvironment simulationProcessEnvironment();
@@ -151,6 +157,8 @@ public:
   static QMap<int, int> getLeadingSpaces(QString contents);
   static int getLeadingSpacesSize(QString str);
   static bool isFileWritAble(QString filePath);
+  static bool containsSpace(QString str);
+  static QString joinDerivativeAndPreviousVariable(QString fullVariableName, QString variableName, QString derivativeOrPrevious);
 protected:
   static QString mLastOpenDir;
 };
