@@ -54,8 +54,6 @@
 #include "cell.h"
 #include "document.h"
 
-class IndentationState;
-
 namespace IAEX
 {
   enum latexCellStates {Finished_l, Eval_l, Error_l, Modified_l};
@@ -94,10 +92,7 @@ namespace IAEX
     void setStatusMenu(QList<QAction*>);
 
   public slots:
-    void eval();
-    void command();
-    void nextCommand();
-    void nextField();
+    void eval(bool silent=false);
     void clickEvent();
     void clickEventOutput();
     void contentChanged();
@@ -116,7 +111,6 @@ namespace IAEX
     virtual void setFocus(const bool focus);
     virtual void setFocusOutput(const bool focus);
     void setExpr(QString);
-    //void showVariableButton(bool);
     void setState(int state);
 
   protected:
@@ -169,19 +163,12 @@ namespace IAEX
     int state;
 
   public slots:
-    void goToPos(const QUrl&);
     void updatePosition();
     void setModified();
-    void indentText();
-    bool lessIndented(QString);
-    void setAutoIndent(bool);
 
   signals:
     void clickOnCell();
     void wheelMove( QWheelEvent* );
-    void command();
-    void nextCommand();
-    void nextField();
     void eval();
     void forwardAction( int );
     void updatePos(int, int);
@@ -197,10 +184,6 @@ namespace IAEX
 
   private:
     bool inCommand;
-    int indentationLevel(QString, bool b=true);
-    bool autoIndent;
-    QMap<int, IndentationState*> indentationStates;
-
   };
 
   class MyAction1: public QAction
